##################
#load packages
library(nat)
library(Rtsne)
library(ggplot2)

#load data
DATADIR<-"~/课题/Work\ report/Code/Data/"
FDIR<-"~/课题/Work\ report/Code/ForFigures/"
FIGUREDIR<-"~/课题/Work\ report/Code/Figures/"
if(!dir.exists(FIGUREDIR)){
  dir.create(FIGUREDIR)
}

load(paste0(DATADIR,"Neurons"))
load(paste0(DATADIR,"Neurondistmatx"))

Neurons[,]$Type<-as.character(Neurons[,]$Type)
Neurons[,]$Type<-as.factor(substr(Neurons[,]$Type,1,2))
##################
#Figure 1c
open3d()
Color<-read.csv(paste0(FDIR,"color.txt"),sep=",",header = F)
plot3d(Neurons,col=as.character(Color$V1[match(Neurons[,]$BrainRegion,as.character(Color$V2))]),add=T)
shade3d(readOBJ(paste0(FDIR,"root.obj")),color="grey",override = T,alpha=0.1)
umatx<-matrix(c(0.9,0.26,-0.36,0,0.08,-0.89,-0.46,0,-0.44,0.38,-0.82,0,0,0,0,1),4,4)
par3d(windowRect = c(103,202,1396,1051),zoom=.7,userMatrix=umatx)
rgl.snapshot(paste0(FIGUREDIR,"Figure 1c.png"))
rgl.close()

##################
#Figure 1d
out.dist<-as.dist(distmatx)
tsne<-Rtsne(out.dist,pca_center=F,is_distance=T)

type<-Neurons[,]$Type[match(rownames(distmatx),names(Neurons))]
texts<-data.frame(dim1=rep(NA,length(unique(type))),dim2=rep(NA,length(unique(type))),text=rep(NA,length(unique(type))))
for(i in levels(type)){
  texts$dim1[which(unique(type)==i)]<-mean(tsne$Y[which(type==i),1])
  texts$dim2[which(unique(type)==i)]<-mean(tsne$Y[which(type==i),2])
  texts$text[which(unique(type)==i)]<-paste0("n=",prettyNum(length(which(type==i)),big.mark = ","))
}

ggplot(data.frame(tSNE1=tsne$Y[,1], tSNE2=tsne$Y[,2],Type=factor(type,levels = c("IT","PT","CT"))))+
  geom_point(aes(x=tSNE1, y=tSNE2, col=Type),size=0.01)+
  guides(colour=guide_legend(override.aes=list(size=1)))+
  annotate("text",x=texts$dim1,y=texts$dim2,label=texts$text,check_overlap=T,size=3)+
  theme_classic()+
  scale_color_manual(values = c("red","green","blue"))+
  labs(title="Classification of PFC neurons based on \nglobal axon alignment")+
  theme(plot.title=element_text(size=10,hjust = 0.5),
        text=element_text(size=8),
        axis.text.x = element_text(size=8),axis.text.y = element_text(size=8),
        axis.title.x = element_text(size=8),axis.title.y = element_text(size=8),
        legend.title = element_text(size=8),legend.text = element_text(size=8))
ggsave(paste0(FIGUREDIR,"Figure 1d.pdf"),device = "pdf",width = 80,height = 60,scale=1,units = "mm")

##################
#Figure 1e
open3d()
mfrow3d(1,length(unique(Neurons[,]$Type)))
par3d(windowRect = c(103,202,1396,1051))
Color<-read.csv(paste0(FDIR,"color.txt"),sep=",",header = F)
for(i in c("IT","PT","CT")){
  next3d()
  plot3d(subset(Neurons,Type==i),col=as.character(Color$V1[match(Neurons[,]$BrainRegion[which(Neurons[,]$Type==i)],as.character(Color$V2))]),add=T)
  shade3d(readOBJ(paste0(FDIR,"root.obj")),color="grey",override = T,alpha=0.1)
  umatx<-matrix(c(0,-1,0,0,0,0,-1,0,1,0,0,0,0,0,0,1),4,4)
  par3d(zoom=.6,userMatrix=umatx)
}

rgl.snapshot(paste0(FIGUREDIR,"Figure 1e.png"))
rgl.close()

##################
#Figure 1f
Color<-c("red","green","blue")
names(Color)<-c("IT","PT","CT")
somapos<-data.frame(X=unlist(lapply(Neurons,function(x)x$d$X[1])),Y=unlist(lapply(Neurons,function(x)x$d$Y[1])),Z=unlist(lapply(Neurons,function(x)x$d$Z[1])))

open3d()
plot3d(somapos,col=Color[match(Neurons[,]$Type,names(Color))],type="s",radius=30,box=F,axes=F,xlab="",ylab="",zlab="")
shade3d(readOBJ(paste0(FDIR,"PFC.obj")),color="grey",override = T,alpha=0.1)
umatx<-matrix(c(-0.08,-0.9,-0.43,0,-0.33,0.43,-0.84,0,0.94,0.08,-0.33,0,0,0,0,1),4,4)
par3d(windowRect = c(103,202,1396,1051),zoom=.7,userMatrix=umatx)
rgl.snapshot(paste0(FIGUREDIR,"Figure 1f.png"))
rgl.close()