% Code for constructing flatmap of isocortex.
% 2020/12/08-Le Gao (lgao@ion.ac.cn)


%% Step 1: construct the mask of Isocortex and delete the boundary.
addpath(genpath('../Matlab_shared/'));
mask_Isocortex = nrrdread('./structure_315_ISO.nrrd');
mask_CLA = nrrdread('./structure_583_CLA.nrrd');

mask_combined = mask_Isocortex + mask_CLA;

boundary = nrrdread('./isocortex_boundary_10.nrrd');
% 
[ROW, COL, SLICE] = size(mask_Isocortex);

% only keep left hemisphere
mask_combined(:,:,SLICE/2+1:SLICE) = 0;

% delete the boundary
mask_combined(boundary > 0) = 0;


%% Step 2: calculate the isosurface of Isocortex mask.
[f, v] = isosurface(mask_combined, 0, 'verbose');


%% Step 3: extract the vertex and face of the superficial layer.
index_superficial = find(boundary == 128);

index_vertex = sub2ind([800, 1320, 1140], v(:,2), v(:,1), v(:,3));

%
NumVertex = size(v, 1);
flag_vertex_superficial = zeros(NumVertex, 1, 'uint8');

parfor iVertex = 1:1:NumVertex
    if(ismember(index_vertex(iVertex, 1), index_superficial))
        flag_vertex_superficial(iVertex, 1) = 1;
    end
    
    if(rem(iVertex, 10000) == 0)
        disp(iVertex);
    end
end

%
NumFace = size(f, 1);
flag_face_superficial = zeros(NumFace, 1, 'uint8');

parfor iFace = 1:1:NumFace
    tmpFace = f(iFace, :);
    
    if(3 == sum(flag_vertex_superficial(tmpFace)))
        flag_face_superficial(iFace, 1) = 1;
    end
end


%% Step 4: construct the graph.
adjacency_matrix = sparse(NumVertex, NumVertex);
for iFace = 1:1:NumFace
        if(flag_face_superficial(iFace) == 1)
                tmpFace = f(iFace, :);
                a = tmpFace(1,1); b = tmpFace(1,2); c = tmpFace(1,3);
                coordinate_a = v(a, :); coordinate_b = v(b, :); coordinate_c = v(c, :);

                dis_ab = sqrt(sum((coordinate_a - coordinate_b).^2));
                dis_ac = sqrt(sum((coordinate_a - coordinate_c).^2));
                dis_bc = sqrt(sum((coordinate_b - coordinate_c).^2));

                adjacency_matrix(a, b) = dis_ab;
                adjacency_matrix(a, c) = dis_ac;
                adjacency_matrix(b, c) = dis_bc;

                adjacency_matrix(b, a) = dis_ab;
                adjacency_matrix(c, a) = dis_ac;
                adjacency_matrix(c, b) = dis_bc;
        end

        if(rem(iFace, 10000) == 0)
                fprintf('%d/%d\n', iFace, NumFace);
        end
end

G = graph(adjacency_matrix);


%% Step 5: define an anchor as the origin and compute geodesic distance 
%  between each voxels and the anchor.
flag_vertex_anchor_center = (v(:,1) == 650) & (v(:,3) == 350) & (flag_vertex_superficial);
index_vertex_anchor_center = find(flag_vertex_anchor_center == 1);


[TP, D] = shortestpathtree(G, 634671);


%% Step 6: compute the coordinate of each voxels on flatmap with orientation and distance maintained.
anchor_center = [650, 350]; % col, slice

coordinate_flatmap = zeros(NumVertex, 2, 'uint16');
image_flatmap = zeros(1500, 1500, 'uint16');

for iVertex = 1:1:NumVertex
    if(flag_vertex_superficial(iVertex, 1) == 0)
        continue;
    end
    
    tmpCol = v(iVertex, 1);
    tmpRow = v(iVertex, 2);
    tmpSlice = v(iVertex, 3);
    
    d = sqrt((anchor_center(1,1) - tmpCol)^2 + (anchor_center(1,2) - tmpSlice)^2);
   
    if(d == 0)
        coordinate_flatmap(iVertex, :) = [0, 0];
        continue;
    end
    
    if(abs(D(1, iVertex)) > 1000)
            continue;
    end
    
    tmpCoordinate = [D(1, iVertex)/d*(tmpSlice - anchor_center(1,2)), ...
                     D(1, iVertex)/d*(tmpCol - anchor_center(1,1))]; 
                                  
    coordinate_flatmap(iVertex, :) = [floor(tmpCoordinate(1,1)) + 800, ...
                                      floor(tmpCoordinate(1,2)) + 800];
                                  
    image_flatmap(coordinate_flatmap(iVertex, 2), ...
                  coordinate_flatmap(iVertex, 1)) = anno_PFC(tmpRow, tmpCol, tmpSlice);

                                  
    if(rem(iVertex, 10000) == 0)
        disp(iVertex);
    end  
end

figure;
plot(coordinate_flatmap(:,1), coordinate_flatmap(:,2), '.')

figure;
imagesc(image_flatmap);

func_matrix2tiff(image_flatmap, 16, './image_flatmap.tiff');

