# IT neurons: level 1
#
# 2020/11/21-Le GAO


library("gplots")
library("fastcluster")
library("ComplexHeatmap")
library("dendextend")
library("colorspace")
library("stringr")
library("nat")
library("amap")

library(ggridges)
library(ggpubr)


setwd("~/Documents/YanLab/Task_Local/Classification_IT_Ipsilateral_and_contralateral_isocortex_striatum/")
source("../shared_R/public.r")

# Read files.
proj = read.table("./PFC/projection_exact_value.txt")
proj_name_neuron = read.table("./PFC/name_neuron.txt")
proj_name_region = read.table("./PFC/name_region.txt")
laminar_neuron = read.table("./PFC/laminar_position.txt")
label_CC_neuron = read.table("./PFC/label_CC.txt")
soma_position = read.table("./PFC/soma.txt")

proj_as_matrix = as.matrix(proj)
rownames(proj_as_matrix) <- as.character(proj_name_neuron[,1])
colnames(proj_as_matrix) <- as.character(proj_name_region[,1])

NumNeuron = dim(proj_as_matrix)[1]
NumTargetRegion = dim(proj_as_matrix)[2]

LabelSourceRegion = func_getLabelRegion(NumNeuron = NumNeuron, proj_name_neuron = proj_name_neuron)
LabelSubtype = func_getLabelSubtype(NumNeuron = NumNeuron, proj_name_neuron = proj_name_neuron)
NeuronDistribution = func_getRegionDistribution(proj_name_neuron[,1])

SWCPath = func_getSWCPath(proj_name_neuron)




## 
proj_combined = array(data = 0, dim = c(NumNeuron, 4))

proj_combined[,1] = rowSums(proj_as_matrix[,seq(1,25)])
proj_combined[,2] = proj_as_matrix[,26]

proj_combined[,3] = rowSums(proj_as_matrix[,seq(27,51)])
proj_combined[,4] = proj_as_matrix[,52]


##
proj_binary = array(data = 0, dim = dim(proj_combined))

Threshold = c(1000, 1000, 1000, 1000)

for(iTargetRegion in 1:4) {
  proj_binary[,iTargetRegion] = ifelse(proj_combined[,iTargetRegion] > Threshold[iTargetRegion], 1, 0)
}

colnames(proj_binary) = c("Isocortex.l", "Striatum.l", "Isocortex.r", "Striatum.r")



## Assign label to each neuron
Label_Cluster_L1 = array(data = 0, dim = c(NumNeuron, 1))
counter_pattern = 0

for(i in 0:1)
  for(j in 0:1)
    for(m in 0:1)
      for(n in 0:1) {
        counter_pattern = counter_pattern + 1
        
        tmpIndexNeuronIpsi = base::intersect(which(proj_binary[,1] == i), 
                                             which(proj_binary[,2] == j))
        
        tmpIndexNeuronContra = base::intersect(which(proj_binary[,3] == m), 
                                               which(proj_binary[,4] == n))
        
        tmpIndexNeuron = base::intersect(tmpIndexNeuronIpsi, tmpIndexNeuronContra)
        
        Label_Cluster_L1[tmpIndexNeuron, 1] = counter_pattern
      }


## 
ratio = sort(table(Label_Cluster_L1), decreasing = T)/dim(proj_as_matrix)[1]
barplot(ratio, ylim = c(0, 0.5))


# this order is the reversed order of the figure
labelOnDend = c(12, 9, 11, 13, 15, 16)


frequency_type = array(data = 0, dim = c(NumSourceRegion, 6))

## Ratio of six types in each PFC subregion
for(iSourceRegion in 1:NumSourceRegion) {
  for(iType in 1:6) {
    frequency_type[iSourceRegion, iType] = 
      length(base::intersect(which(LabelSourceRegion == SourceRegion[iSourceRegion]), 
                             which(Label_Cluster_L1 == labelOnDend[iType])))
  }
}

ratio_type = apply(frequency_type, 1, function(x) x/sum(x))
barplot(ratio_type, col = rev(ColorCluster[1:6]), names.arg = SourceRegion)
#legend("topright", ColorCluster[1:6], lty = 1)


##########

NumCluster = 6

## Laminar position for each cluster
position = c()
group = c()
type = c()

xlabel = c("L1","L2/3","L5","L6a","L6b")

counterGroup = 0;
for (iCluster in 1:NumCluster) {
  tmpPosition = laminar_neuron$V1[which(labelOnDend[iCluster] == Label_Cluster_L1)]
  
  counterGroup = counterGroup + 1
  group = c(group, rep(counterGroup, length(tmpPosition) + NumNeuron))
  type = c(type, rep(counterGroup, length(tmpPosition)), rep(0, NumNeuron))
  position = c(position, tmpPosition, laminar_neuron$V1)
}


test = data.frame(group = as.factor(group),
                  type = as.factor(type),
                  position = position)

ggplot(test, aes(x = position, y = group, color = type, fill = type)) +
  geom_density_ridges(scale = 0.9,
                      size = 0.1, 
                      jittered_points = FALSE,
                      position = position_points_jitter(width = 0.05, height = 0),
                      point_shape = '|', point_size = 3, point_alpha = 1, alpha = 0.3)  +
  #scale_x_continuous(expand = c(0.01,0.7)) +
  #scale_y_discrete(expand = c(0.01,0.7), limits = rev(levels(as.factor(group)))) +
  #scale_fill_manual(values = color, labels = seq(1, 14, by = 1)) +
  theme_ridges(grid = FALSE)+
  theme(axis.text = element_text(size = 7), line = element_line(size = 0.25))+
  xlim(c(0,1)) +
  xlab(NULL) +
  ylab(NULL)
#scale_fill_cyclical(values = clusterColorList)
