# IT neurons: ipsilateral and contralateral projection pattern
# hemotopic projection or else
#
# 2020/10/11-Le GAO


library("gplots")
library("fastcluster")
library("ComplexHeatmap")
library("dendextend")
library("colorspace")
library("stringr")
library("nat")
library("amap")

library(ggridges)
library(ggpubr)


setwd("~/Documents/YanLab/Task_Local/Classification_IT_Ipsilateral_and_contralateral_isocortex_striatum/")
#source("../shared_R/public.r")

# Read files.
proj = read.table("./PFC/projection_exact_value.txt")
proj_name_neuron = read.table("./PFC/name_neuron.txt")
proj_name_region = read.table("./PFC/name_region.txt")
laminar_neuron = read.table("./PFC/laminar_position.txt")
label_CC_neuron = read.table("./PFC/label_CC.txt")
soma_position = read.table("./PFC/soma.txt")

proj_as_matrix = as.matrix(proj)
rownames(proj_as_matrix) <- as.character(proj_name_neuron[,1])
colnames(proj_as_matrix) <- as.character(proj_name_region[,1])

NumNeuron = dim(proj_as_matrix)[1]
NumTargetRegion = dim(proj_as_matrix)[2]

LabelSourceRegion = func_getLabelRegion(NumNeuron = NumNeuron, proj_name_neuron = proj_name_neuron)
LabelSubtype = func_getLabelSubtype(NumNeuron = NumNeuron, proj_name_neuron = proj_name_neuron)
NeuronDistribution = func_getRegionDistribution(proj_name_neuron[,1])

SWCPath = func_getSWCPath(proj_name_neuron)



## 
proj_combined = array(data = 0, dim = c(NumNeuron, 4))

proj_combined[,1] = rowSums(proj_as_matrix[,c(1,2,3,4,5,6,7,8,9,17,18)])
proj_combined[,2] = rowSums(proj_as_matrix[,c(10,11,12,13,14,15,16,19,20,21,22,23,24,25)])

proj_combined[,3] = rowSums(proj_as_matrix[,26+c(1,2,3,4,5,6,7,8,9,17,18)])
proj_combined[,4] = rowSums(proj_as_matrix[,26+c(10,11,12,13,14,15,16,19,20,21,22,23,24,25)])


##
proj_binary = array(data = 0, dim = dim(proj_combined))

Threshold = c(1000, 10000, 1000, 10000)

for(iTargetRegion in 1:4) {
  proj_binary[,iTargetRegion] = ifelse(proj_combined[,iTargetRegion] > Threshold[iTargetRegion], 1, 0)
}

colnames(proj_binary) = c("PFC.l", "Non-PFC.l", "PFC.r", "Non-PFC.r")


## Assign label to each neuron
Label_Cluster_L2 = array(data = 0, dim = c(NumNeuron, 1))
counter_pattern = 0

for(i in 0:1)
  for(j in 0:1)
    for(m in 0:1)
      for(n in 0:1) {
            counter_pattern = counter_pattern + 1
            
            tmpIndexNeuronIpsi = base::intersect(which(proj_binary[,1] == i), 
                                                 which(proj_binary[,2] == j))
            
            tmpIndexNeuronContra = base::intersect(which(proj_binary[,3] == m), 
                                                   which(proj_binary[,4] == n))
            
            tmpIndexNeuron = base::intersect(tmpIndexNeuronIpsi, tmpIndexNeuronContra)
            
            Label_Cluster_L2[tmpIndexNeuron, 1] = counter_pattern
          }

# write index of neurons for level-3
index_neuron_13 = which(Label_Cluster_L2 == 13)
index_neuron_15 = which(Label_Cluster_L2 == 15)
index_neuron_16 = which(Label_Cluster_L2 == 16)

label = c(replicate(length(index_neuron_13), 13),
          replicate(length(index_neuron_15), 15),
          replicate(length(index_neuron_16), 16))

index_level_3 = array(data = 0, dim = c(length(label), 2))
index_level_3[,1] = c(index_neuron_13, index_neuron_15, index_neuron_16)
index_level_3[,2] = label
colnames(index_level_3) = c("index", "label_L2")
                      
write.table(index_level_3, file = "./index_level_3_10000.csv")



# this order is the reversed order of the figure
labelOnDend = c(16, 12, 15, 11, 13, 9)


frequency_type = array(data = 0, dim = c(NumSourceRegion, 6))

## Ratio of six types in each PFC subregion
for(iSourceRegion in 1:NumSourceRegion) {
  for(iType in 1:6) {
    frequency_type[iSourceRegion, iType] = 
      length(base::intersect(which(LabelSourceRegion == SourceRegion[iSourceRegion]), 
                             which(Label_Cluster_L2 == labelOnDend[iType])))
  }
}

ratio_type = apply(frequency_type, 1, function(x) x/sum(x))
barplot(ratio_type, col = rev(ColorCluster[1:6]), names.arg = SourceRegion)
legend("topright", ColorCluster[1:6], lty = 1)

##

NumCluster = 6

## Laminar position for each cluster
position = c()
group = c()
type = c()

xlabel = c("L1","L2/3","L5","L6a","L6b")

counterGroup = 0;
for (iCluster in 1:NumCluster) {
  tmpPosition = laminar_neuron$V1[which(labelOnDend[iCluster] == Label_Cluster_L2)]
  
  counterGroup = counterGroup + 1
  group = c(group, rep(counterGroup, length(tmpPosition) + NumNeuron))
  type = c(type, rep(counterGroup, length(tmpPosition)), rep(0, NumNeuron))
  position = c(position, tmpPosition, laminar_neuron$V1)
}


test = data.frame(group = as.factor(group),
                  type = as.factor(type),
                  position = position)

ggplot(test, aes(x = position, y = group, color = type, fill = type)) +
  geom_density_ridges(scale = 0.9,
                      size = 0.1, 
                      jittered_points = FALSE,
                      position = position_points_jitter(width = 0.05, height = 0),
                      point_shape = '|', point_size = 3, point_alpha = 1, alpha = 0.3)  +
  #scale_x_continuous(expand = c(0.01,0.7)) +
  #scale_y_discrete(expand = c(0.01,0.7), limits = rev(levels(as.factor(group)))) +
  #scale_fill_manual(values = color, labels = seq(1, 14, by = 1)) +
  theme_ridges(grid = FALSE)+
  theme(axis.text = element_text(size = 7), line = element_line(size = 0.25))+
  xlim(c(0,1)) +
  xlab(NULL) +
  ylab(NULL)
#scale_fill_cyclical(values = clusterColorList)





### SM7 heatmap for ITc neurons.

index_neuron_ITc_11_12 = base::union(which(Label_Cluster_L2 == 11), 
                               which(Label_Cluster_L2 == 12))
index_neuron_ITc_15_16 = base::union(which(Label_Cluster_L2 == 15), 
                                     which(Label_Cluster_L2 == 16))
index_neuron_ITc_11_12_15_16 = base::union(index_neuron_ITc_11_12, index_neuron_ITc_15_16)



ht = Heatmap(matrix = proj_as_matrix[index_neuron_ITc_11_12_15_16, 
                                     c(seq(1,25,1), seq(27,51,1))], 
             name = "Normalized projection strength",
             heatmap_legend_param	= list(title_gp = gpar(fontsize = 5),
                                         labels_gp = gpar(fontsize = 5)),
             # 
             cluster_rows = TRUE,
             clustering_distance_rows = "spearman",
             clustering_method_rows = "ward.D2",
             # 
             cluster_columns = FALSE,
             clustering_distance_columns = "euclidean",
             clustering_method_columns = "ward.D2",
             
             
             column_split = c(replicate(25, "0Ipsi-"), replicate(25, "1Contral-")),
             cluster_column_slices = FALSE,
             
             #top_annotation = column_ha,
             #cluster_rows = rowDend,
             row_split = LabelSourceRegion[index_neuron_ITc_11_12_15_16],
             cluster_row_slices = FALSE,
             #row_split = 6,
             
             #row_dend_side = "left",
             
             #column_split = LabelSourceRegion,
             #cluster_column_slices = FALSE,
             #column_split = LabelSubtype[which(LabelSourceRegion == "ACAd")],
             #row_title = "IT_%s",
             row_title_gp = gpar(fontsize = 7),
             # row_title_side = "right",
             row_title_rot = 0,
             
             width = unit(12, "cm"), height = unit(15, "cm"),
             
             show_row_names = FALSE,
             row_names_side = "right", row_names_rot = 0, row_names_gp = gpar(fontsize = 6),
             column_names_side = "top", column_names_rot = 90, column_names_gp = gpar(fontsize = 6),
             use_raster = TRUE
             )
draw(ht)



## Symmetry of projection pattern for PFC subregions
# 
# index_coverage = array(data = 0, dim = c(NumSourceRegion, 1))
# index_symmetry = array(data = 0, dim = c(NumSourceRegion, 1))
# 
# for(iSourceRegion in 1:NumSourceRegion) {
#   tmpIndex = base::intersect(which(LabelSourceRegion == SourceRegion[iSourceRegion]),
#                              index_neuron_ITc_11_12_15_16)
#   
#   tmpNum = length(tmpIndex)
#   
#   for(iNeuron in 1:tmpNum) {
#     
#     tmpNumRegion_left = length(which(proj_as_matrix[tmpIndex[iNeuron], seq(1,25,1)] > 0))
#     tmpNumRegion_right = length(which(proj_as_matrix[tmpIndex[iNeuron], seq(27,51,1)] > 0))
#     
#     index_area[iNeuron, 1] = tmpNumRegion_left/(tmpNumRegion_left + tmpNumRegion_right)
#     
#     index_symmetry[iNeuron, 1] = Dist(rbind(t(proj_as_matrix[tmpIndex[iNeuron], seq(1,25,1)]),
#                                             t(proj_as_matrix[tmpIndex[iNeuron], seq(27,51,1)])),
#                                       method = "pearson")
#   }
# }


index_coverage = array(data = 0, dim = c(length(index_neuron_ITc_11_12_15_16), 1))
index_symmetry = array(data = 0, dim = c(length(index_neuron_ITc_11_12_15_16), 1))
for(iNeuron in 1:length(index_neuron_ITc_11_12_15_16)) {
  
  tmpProjIpsi = proj_as_matrix[index_neuron_ITc_11_12_15_16[iNeuron], seq(1,25,1)]
  tmpProjContra = proj_as_matrix[index_neuron_ITc_11_12_15_16[iNeuron], seq(27,51,1)]
  
  tmpNumRegion_left = length(which(tmpProjIpsi > 0))
  tmpNumRegion_right = length(which(tmpProjContra > 0))
  
  index_coverage[iNeuron, 1] = tmpNumRegion_left/(tmpNumRegion_left + tmpNumRegion_right)
  
  index_symmetry[iNeuron, 1] = 1-Dist(rbind(t(tmpProjIpsi), t(tmpProjContra)), method = "pearson")
}



df_coverage = data.frame(value = index_coverage, 
                         group = as.factor(LabelSourceRegion[index_neuron_ITc_11_12_15_16]))


df_symmetry = data.frame(value = index_symmetry,
                         group = as.factor(LabelSourceRegion[index_neuron_ITc_11_12_15_16]))


ggboxplot(data = df_symmetry, x = "group", y = "value", orientation = "horizontal", 
          order = rev(c("ORBvl", "ORBm", "ORBl", "FRP", "PL", 
                    "ACAd", "AIv", "ACAv", "ILA", "MOs", "AId"))) +
  stat_compare_means(label = "p.signif", method = "wilcox.test", ref.group = ".all.") +
  geom_hline(yintercept = mean(df_symmetry$value))



ggboxplot(data = df_symmetry, x = "group", y = "value", orientation = "horizontal") +
  stat_compare_means(label = "p.signif", method = "t.test", ref.group = ".all.") +
  geom_hline(yintercept = mean(df_symmetry$value))



#plot(0,0, add = TRUE)
abline(v = mean(df_symmetry$value), add = TRUE)





ACAd_index_neuron_ITc = base::intersect(which(LabelSourceRegion == "ACAd"), which(LabelSubtype == "ITc"))

NumNeuronACAdITc = length(ACAd_index_neuron_ITc)

index_area = array(data = 0, dim = c(NumNeuronACAdITc, 1))
index_symmetry = array(data = 0, dim = c(NumNeuronACAdITc, 1))

for(iNeuron in 1:NumNeuronACAdITc) {
  tmpNumRegion_left = length(which(proj_as_matrix[ACAd_index_neuron_ITc[iNeuron], 1:5] > 0))
  tmpNumRegion_right = length(which(proj_as_matrix[ACAd_index_neuron_ITc[iNeuron], 6:10] > 0))
  
  index_area[iNeuron, 1] = tmpNumRegion_left/(tmpNumRegion_left + tmpNumRegion_right)
  
  index_symmetry[iNeuron, 1] = Dist(rbind(t(proj_as_matrix[ACAd_index_neuron_ITc[iNeuron], 1:5]),
                                          t(proj_as_matrix[ACAd_index_neuron_ITc[iNeuron], 6:10])),
                                    method = "pearson")
}












## put neurons together and general subtype
columnDend = column_dend(ht)
columnOrder = column_order(ht)


NumCluster = 5

Label_Cluster = cutree(tree = columnDend, k = NumCluster)

labelOnDend = unique(Label_Cluster[columnOrder])


## Laminar position for each cluster
position = c()
group = c()
type = c()

xlabel = c("L1","L2/3","L5","L6a","L6b")

counterGroup = 0;
for (iCluster in 1:NumClusters) {
  tmpPosition = laminar_neuron$V1[which(labelOnDend[iCluster] == Label_Cluster)]
  
  counterGroup = counterGroup + 1
  group = c(group, rep(counterGroup, length(tmpPosition) + NumNeuron))
  type = c(type, rep(counterGroup, length(tmpPosition)), rep(0, NumNeuron))
  position = c(position, tmpPosition, laminar_neuron$V1)
}


test = data.frame(group = as.factor(group),
                  type = as.factor(type),
                  position = position)

ggplot(test, aes(x = position, y = group, color = type, fill = type)) +
  geom_density_ridges(scale = 0.9,
                      size = 0.1, 
                      jittered_points = FALSE,
                      position = position_points_jitter(width = 0.05, height = 0),
                      point_shape = '|', point_size = 3, point_alpha = 1, alpha = 0.3)  +
  #scale_x_continuous(expand = c(0.01,0.7)) +
  scale_y_discrete(expand = c(0.01,0.7), limits = rev(levels(as.factor(group)))) +
  #scale_fill_manual(values = color, labels = seq(1, 14, by = 1)) +
  theme_ridges(grid = FALSE)+
  theme(axis.text = element_text(size = 7), line = element_line(size = 0.25))+
  xlim(c(0,1)) +
  xlab(NULL) +
  ylab(NULL)
  #scale_fill_cyclical(values = clusterColorList)


ggsave2("/home/admin/shared/figure5/laminar_position.pdf", 
        scale = 1, width = 30, height = 40, units = "mm", dpi = 300)


## statistical test
sta_position = c()
sta_group = c()

counterGroup = 0;
for (iCluster in 1:NumClusters) {
  tmpPosition = laminar_neuron$V1[which(iCluster == clusters)]
  
  counterGroup = counterGroup + 1
  sta_group = c(sta_group, rep(counterGroup, length(tmpPosition)))
  sta_position = c(sta_position, tmpPosition)
}

sta_group = c(sta_group, rep(0, NumNeuron))
sta_position = c(sta_position, laminar_neuron$V1)

sta_laminarPosition = data.frame(sta_group = as.factor(sta_group),
                                 sta_position = sta_position)


xx=ggpubr::compare_means(formula = sta_position~sta_group, 
                      data = sta_laminarPosition, 
                      methods = "kruskal.test",
                      ref.group = "0", 
                      p.adjust.method = "none")


## Calculation of symmetry for each cluster
symmetry_value = array(0, dim = c(NumNeuron, 1))
symmetry_group = array(0, dim = c(NumNeuron, 1))

for (iCluster in 1:NumClusters) {
  tmpNeuronIndexList = which(iCluster == clusters)
  
  for (iNeuron in 1:length(tmpNeuronIndexList)) {
    tmpNeuronIndex = tmpNeuronIndexList[iNeuron]
    
    tmpLeft = as.numeric(proj_as_matrix[tmpNeuronIndex,1:(NumRegion/2)] > 0)
    tmpRight = as.numeric(proj_as_matrix[tmpNeuronIndex, (NumRegion/2+1):NumRegion] > 0)
    
    tmpIntersection = as.numeric(tmpLeft & tmpRight)
    tmpUnion = as.numeric(tmpLeft | tmpRight)
    
    tmpJaccard = length(which(tmpIntersection == 1)) / length(which(tmpUnion == 1))
    #tmpCor = cor(tmpLeft, tmpRight)
    
    symmetry_value[tmpNeuronIndex,1] = tmpJaccard
    symmetry_group[tmpNeuronIndex,1] = iCluster
  }
}

symmetry = data.frame(cluster = as.factor(symmetry_group), value = symmetry_value)

ggboxplot(symmetry, x = "cluster", y = "value", orientation = "horizontal") +
  scale_x_discrete(limits = rev(levels(as.factor(symmetry_group)))) +
  stat_compare_means(label = "p.signif", method = "t.test", ref.group = ".all.")



  
ggplot(symmetry, aes(cluster, value)) +
  geom_boxplot()


## Choose number of clusters
dist = Dist(x = proj_as_matrix, method = "euclidean")

## Determine the number of clusters
nc = NbClust(data = proj_as_matrix, 
             diss = dist, 
             distance = NULL,
             min.nc = 4,
             max.nc = 12, 
             method = "ward.D2",
             index = "all"
)

nc$All.index[,"Pseudot2"]
nc$Best.partition


## Write clusters and name of neurons (Sang LIU)
out_cluster = as.matrix(clusters)
rownames(out_cluster) = proj_name_neuron[,1]

write.table(x = out_cluster, 
            file = "/tmp/classification_IT.csv", 
            sep = ",", 
            row.names = TRUE, 
            col.names = FALSE)

## PCA
library(FactoMineR)
pcaaa = PCA(proj_as_matrix[base::intersect(which(LabelSourceRegion == "ACAd"), 
                                                which(LabelSubtype == "ITc")), 1:25],
            scale.unit = TRUE, ncp = 25, graph = T)

loadings = pcaaa$loadings

pca = prcomp(x = proj_as_matrix, scale. = TRUE)
sdev = pca$sdev
sdev/sum(sdev^2)
pca$sdev



# One neuron
index_neuron = base::intersect(which(Label_Cluster_L2 == 12),
                               which(LabelSourceRegion == "ACAd"))[4]
#index_neuron = 2134
swc = read.neuron(SWCPath[index_neuron])

#swc = read.neuron("/home/admin/Documents/YanLab/database_neuron/195199/swc_allen_space/019.swc")

swc = func_flipRight2Left(as.neuronlist(swc), proj_name_neuron[index_neuron,1])
swc = swc[[1]]

print(proj_name_neuron[index_neuron,1])
#segments_Isocortex = func_getSegments(swc, pointsinside(swc, mesh_Isocortex))


open3d()
#segments3d(segments_Isocortex, col = "red", lwd = 2)
plot3d(swc, WithNodes = FALSE, col = "grey", soma = 100, lwd = 2)
shade3d(mesh_root, col = "grey", alpha = 0.1)
view3d(userMatrix = userMatrix_IT_DV, fov = 0)


## visualization (population)
index_neuron = base::intersect(which(Label_Cluster_L2 == 15),
                               which(LabelSourceRegion == "ORBl"))
print(proj_name_neuron[index_neuron,1])

test_neuronlist = read.neurons(paths = SWCPath[index_neuron], 
                               neuronnames = proj_name_neuron[index_neuron,1])

test_neuronlist_flipped = func_flipRight2Left(test_neuronlist, proj_name_neuron[index_neuron,1])

open3d()
plot3d(test_neuronlist_flipped)
shade3d(mesh_root, col = "grey", alpha = 0.1)
view3d(userMatrix = userMatrix_IT_DV, fov = 0)


