
clear, clc

p = genpath('../Matlab_shared/');
addpath(p);


%% Retrieve data from local database.
javaclasspath('../Matlab_shared/postgresql-42.2.1.jre7.jar');

props = java.util.Properties;
props.setProperty('user', 'projectome');
props.setProperty('password', 'projectome');

driver = org.postgresql.Driver;
url = 'jdbc:postgresql://localhost:5432/projectome';
conn = driver.connect(url, props);

query = ['SELECT tb_neuron.sampleid, tb_neuron.index, brainregion, laminarposition, hemisphere, type ', ...
    'FROM tb_neuron LEFT JOIN tb_projection_hemisphere_v2 ON tb_neuron.sampleid = ' ...
    'tb_projection_hemisphere_v2.sampleid AND tb_neuron.index = tb_projection_hemisphere_v2.index ' ...
    'WHERE type in (''ITi'', ''ITc'') AND exclude = ''0'' ' ...
    'AND brainregion in (''MOs'',''ACAd'',''ACAv'',''PL'',''ILA'',''FRP'',''ORBm'',''ORBvl'',''ORBl'',''AId'',''AIv'') ' ...
    'AND (l_315 > 0 OR r_315 > 0)'];

ps = conn.prepareStatement(query);
rs = ps.executeQuery();

count = 0;
result = struct;

while(rs.next())
    count = count + 1;
    result(count).SampleID = rs.getString(1);
    result(count).Index = rs.getInt(2);
    result(count).BrainRegion = rs.getString(3);
    result(count).LaminarPosition = rs.getString(4);
    result(count).Hemisphere = rs.getString(5);
    result(count).Type = rs.getString(6);
end

rs.close();


%% Calculate projection pattern for each neuron.
%mask_type = 'PL.PT';
METRIC = 'LengthOfAxon'; % 'NumberOfEndNodes'
%[mask, NUM_REGION, NAME_REGION] = func_loadMask(mask_type);

mask = nrrdread('./result/label_PFC_volume_filled_25um.nrrd');
RESOLUTION = 25;
NUM_COLUMN = 40;
NAME_REGION = cell(1, NUM_COLUMN);

for iRegion = 1:1:NUM_COLUMN
    NAME_REGION{1, iRegion} = num2str(iRegion);
end


%%
NUM_NEURON = count;
matrix_projection = zeros(NUM_NEURON, NUM_COLUMN);
matrix_soma = zeros(NUM_NEURON, 3);
NAME_NEURON = cell(NUM_NEURON, 1);
label_neuron = zeros(NUM_NEURON, 1); % corresponds to kCluster

DirPath = '/home/admin/Documents/YanLab/database_neuron/';

%parpool(8)
parfor iNeuron = 1:1:NUM_NEURON
    
    TmpSampleID = char(result(iNeuron).SampleID);
    TmpIndex = num2str(result(iNeuron).Index, '%03d');
    TmpBrainRegion = char(result(iNeuron).BrainRegion);
    TmpHemisphere = char(result(iNeuron).Hemisphere);
    
    SWCPath = strcat(DirPath, TmpSampleID, '/swc_allen_space/', TmpIndex, '.swc');
    swc_allen = dlmread(SWCPath);
    
    % Actually... only work for computing heatmap of IT neurons.
    if strcmp(TmpHemisphere, 'Right')
        swc_allen(:,5) = 456*25 - swc_allen(:,5);
    end
    
    % Fetch x, y, and z coordinates of soma.
    %matrix_soma(iNeuron, :) = swc_allen(1,3:5);
    
    % Label of neuron
    
    tmp_column_anno = floor(swc_allen(1,3) / RESOLUTION) + 1;
    tmp_row_anno = floor(swc_allen(1,4) / RESOLUTION) + 1;
    tmp_slice_image = floor(swc_allen(1,5) / RESOLUTION) + 1;
    
%     tmp_column_anno = floor(matrix_soma(iNeuron,1) / RESOLUTION) + 1;
%     tmp_row_anno = floor(matrix_soma(iNeuron,2) / RESOLUTION) + 1;
%     tmp_slice_image = floor(matrix_soma(iNeuron,3) / RESOLUTION) + 1;
    
    label_neuron(iNeuron, 1) = mask(tmp_row_anno, tmp_column_anno, tmp_slice_image);
    
    % Projection
    [tmp_data, type] = func_calProjectionPattern(swc_allen, RESOLUTION, mask, METRIC, 0);
    
    matrix_projection(iNeuron, :) = tmp_data';
    
    NAME_NEURON{iNeuron, 1} = strcat(TmpSampleID, '+', TmpIndex, '+', TmpBrainRegion);
    
    
    if(0 == rem(iNeuron, 100))
        disp(strcat(num2str(iNeuron),'/',num2str(NUM_NEURON)));
    end
end


%% Filter out neurons with projection strength less than a threshold
THRESHOLD = 1000; % um
vector_projection_sum = sum(matrix_projection, 2);

index_neuron = find(vector_projection_sum > THRESHOLD);


%%
matrix_connection = zeros(NUM_COLUMN, NUM_COLUMN);
distribution_neuron = zeros(NUM_COLUMN, 1);

for iNeuron = 1:1:NUM_NEURON
    if(~ismember(iNeuron, index_neuron))
        continue;
    end
    
    if(label_neuron(iNeuron, 1) == 0)
        continue;
    end
    
    distribution_neuron(label_neuron(iNeuron, 1), 1) = distribution_neuron(label_neuron(iNeuron, 1), 1) + 1;
    
    matrix_connection(label_neuron(iNeuron, 1), :) = matrix_projection(iNeuron, :) ...
        + matrix_connection(label_neuron(iNeuron, 1), :);
    % Set self-connection to 0.
    matrix_connection(label_neuron(iNeuron, 1), label_neuron(iNeuron, 1)) = 0;
end

% Average projection strength
% TODO what if no neurons in a cube.
NUM_COLUMN = 40;
matrix_connection_normalized_raw = matrix_connection./repmat(distribution_neuron, 1, NUM_COLUMN);
matrix_connection_normalized_raw(isnan(matrix_connection_normalized_raw)) = 1; 

figure;
imagesc(matrix_connection_normalized_raw, [0, 2000]);
set(gca, 'XTick', 1:1:NUM_COLUMN)
set(gca, 'YTick', 1:1:NUM_COLUMN)



%% Compare projection strength among domains and across domains

% Projection-defined domains
Label_domain = [1,2;2,1;6,3;7,1;9,2;
                 10,1;11,3;12,3;13,3;14,3;15,2;16,2;18,3;19,2;
                 20,1;22,2;23,1;24,1;25,2;26,2;28,3;
                 30,2;32,1;33,1;34,2;35,3;36,1;37,1;38,1;39,3;40,3];

% Connection-defined modules
Label_module = [1,2,3,1,1,2,3,3,1,3,1,1,3,1,2,1,2,2,1,3,3,1,2,2,1,3,2,2,2,3,3]; 


WithInModule = zeros(31, 1);
for iColumn = 1:1:31
    tmpModule = Label_module(1, iColumn);
    
    tmpConnection = ...
        sum(matrix_connection_normalized_exclude(iColumn, find(Label_module == tmpModule)), 2);
    
    WithInModule(iColumn, 1) = tmpConnection;
end

zDimension = zeros(31,1);
for iColumn = 1:1:31
    
    tmpWholeModule = WithInModule(Label_module==Label_module(1, iColumn),1);
    
    zDimension(iColumn, 1) = (WithInModule(iColumn, 1) - mean(tmpWholeModule)) / std(tmpWholeModule);
end


pDimension = zeros(31,1);
for iColumn = 1:1:31
    tmpModule = Label_module(1, iColumn);
    tmpConnectionTotal = sum(matrix_connection_normalized_exclude(iColumn,:),2);
    tmpP = 1;
    for iModule = 1:1:3
%         if(iModule == tmpModule)
%             continue;
%         end
        
        tmpP = tmpP - (sum(matrix_connection_normalized_exclude(iColumn, find(Label_module == iModule)),2))^2/tmpConnectionTotal^2;
    end
    
    pDimension(iColumn, 1) = tmpP;
end


ColorModule = zeros(31,3);
for iColumn = 1:1:31
    switch Label_module(1,iColumn)
        case 1
            ColorModule(iColumn, :) = [0,1,1];
        case 2
            ColorModule(iColumn, :) = [1,0,1];
        case 3
            ColorModule(iColumn, :) = [1,1,0];
    end
end

figure;
scatter(pDimension, zDimension, ...
        500, ColorModule, 'filled')

text(pDimension, zDimension, num2str(Label_domain(:,1)), 'FontSize',14)
xlabel('Partipation coefficient');
ylabel('With-in module strength');


index_hub = find(pDimension > 0.45);

NumHub = length(index_hub);

connection_hub = zeros(NumHub, NumHub);

for iSourceColumn = 1:1:NumHub
    for iTargetColumn = 1:1:NumHub
        connection_hub(iSourceColumn, iTargetColumn) = ...
            (matrix_connection_normalized_exclude(index_hub(iSourceColumn), index_hub(iTargetColumn)) ...
            + matrix_connection_normalized_exclude(index_hub(iTargetColumn), index_hub(iSourceColumn)));
    end
end

[row, col] = ind2sub([21,21], find(tril(connection_hub) > 5000));

Label_domain(row, 1)


matrix_connection_module = zeros(NUM_COLUMN, 3);


index_module_1 = find(Label_module == 1); % cyan
index_module_2 = find(Label_module == 2); % magenta
index_module_3 = find(Label_module == 3); % yellow

matrix_connection_module_1 = zeros(length(index_module_1), 5); % 1-1, 1-2, 1-3, 1-2/1-1, 1-3/1-1

for iSourceColumn = 1:1:length(index_module_1)
    matrix_connection_module_1(iSourceColumn,1) = ...
        sum(matrix_connection_normalized_exclude(index_module_1(iSourceColumn), index_module_1));
    
    matrix_connection_module_1(iSourceColumn,2) = ...
        sum(matrix_connection_normalized_exclude(index_module_1(iSourceColumn), index_module_2));
    
    matrix_connection_module_1(iSourceColumn,3) = ...
        sum(matrix_connection_normalized_exclude(index_module_1(iSourceColumn), index_module_3));
end

matrix_connection_module_1(:, 4) = matrix_connection_module_1(:,2)./matrix_connection_module_1(:,1);
matrix_connection_module_1(:, 5) = matrix_connection_module_1(:,3)./matrix_connection_module_1(:,1);

        

% Module 2
matrix_connection_module_2 = zeros(length(index_module_2), 5); % 2-2, 2-1, 2-3, 2-1/2-2, 2-3/2-2

for iSourceColumn = 1:1:length(index_module_2)
    matrix_connection_module_2(iSourceColumn,1) = ...
        sum(matrix_connection_normalized_exclude(index_module_2(iSourceColumn), index_module_2));
    
    matrix_connection_module_2(iSourceColumn,2) = ...
        sum(matrix_connection_normalized_exclude(index_module_2(iSourceColumn), index_module_1));
    
    matrix_connection_module_2(iSourceColumn,3) = ...
        sum(matrix_connection_normalized_exclude(index_module_2(iSourceColumn), index_module_3));
end

matrix_connection_module_2(:, 4) = matrix_connection_module_2(:,2)./matrix_connection_module_2(:,1);
matrix_connection_module_2(:, 5) = matrix_connection_module_2(:,3)./matrix_connection_module_2(:,1);

        


% Module 3
matrix_connection_module_3 = zeros(length(index_module_3), 5); % 3-3, 3-1, 3-2, 3-1/3-3, 3-2/3-3

for iSourceColumn = 1:1:length(index_module_3)
    matrix_connection_module_3(iSourceColumn,1) = ...
        sum(matrix_connection_normalized_exclude(index_module_3(iSourceColumn), index_module_3));
    
    matrix_connection_module_3(iSourceColumn,2) = ...
        sum(matrix_connection_normalized_exclude(index_module_3(iSourceColumn), index_module_1));
    
    matrix_connection_module_3(iSourceColumn,3) = ...
        sum(matrix_connection_normalized_exclude(index_module_3(iSourceColumn), index_module_2));
end

matrix_connection_module_3(:, 4) = matrix_connection_module_3(:,2)./matrix_connection_module_3(:,1);
matrix_connection_module_3(:, 5) = matrix_connection_module_3(:,3)./matrix_connection_module_3(:,1);

        
%%
matrix_connection_M1_M2 = matrix_connection_normalized_exclude(index_module_1, :);
matrix_connection_M1_M2 = matrix_connection_M1_M2(:, index_module_2);
sum_M1_M2 = sum(matrix_connection_M1_M2, 2);
Label_domain(index_module_1(1, sum_M1_M2 == max(sum_M1_M2)),1)

matrix_connection_M2_M1 = matrix_connection_normalized_exclude(index_module_2, :);
matrix_connection_M2_M1 = matrix_connection_M2_M1(:, index_module_1);
sum_M2_M1 = sum(matrix_connection_M2_M1, 2);
Label_domain(index_module_2(1, sum_M2_M1 == max(sum_M2_M1)),1)

matrix_connection_M2_M3 = matrix_connection_normalized_exclude(index_module_2, :);
matrix_connection_M2_M3 = matrix_connection_M2_M3(:, index_module_3);
sum_M2_M3 = sum(matrix_connection_M2_M3, 2);
Label_domain(index_module_2(1, sum_M2_M3 == max(sum_M2_M3)),1)


matrix_connection_M1_M3 = matrix_connection_normalized_exclude(index_module_1, :);
matrix_connection_M1_M3 = matrix_connection_M1_M3(:, index_module_3);
sum_M1_M3 = sum(matrix_connection_M1_M3, 2);
Label_domain(index_module_1(1, sum_M1_M3 == max(sum_M1_M3)),1)

matrix_connection_M3_M1 = matrix_connection_normalized_exclude(index_module_3, :);
matrix_connection_M3_M1 = matrix_connection_M3_M1(:, index_module_1);
sum_M3_M1 = sum(matrix_connection_M3_M1, 2);
Label_domain(index_module_3(1, sum_M3_M1 == max(sum_M3_M1)),1)

matrix_connection_M3_M2 = matrix_connection_normalized_exclude(index_module_3, :);
matrix_connection_M3_M2 = matrix_connection_M3_M2(:, index_module_2);
sum_M3_M2 = sum(matrix_connection_M3_M2, 2);
Label_domain(index_module_3(1, sum_M3_M2 == max(sum_M3_M2)),1)



test_sum = matrix_connection_M1_M2 + matrix_connection_M2_M1';
figure;
imagesc(test_sum)
set(gca, 'XTick', 1:1:length(index_module_2), 'XTickLabel', Label_domain(index_module_2,1))
set(gca, 'YTick', 1:1:length(index_module_1), 'YTickLabel', Label_domain(index_module_1,1))
axis equal



test_sum = matrix_connection_M2_M3 + matrix_connection_M3_M2';
figure;
imagesc(test_sum)
set(gca, 'XTick', 1:1:length(index_module_3), 'XTickLabel', Label_domain(index_module_3,1))
set(gca, 'YTick', 1:1:length(index_module_2), 'YTickLabel', Label_domain(index_module_2,1))
axis equal



for iSourceColumn = 1:1:NUM_COLUMN
    tmpLabelModule = Label_module(1, iSourceColumn);
    tmpIndexColumnInSameModule = find(Label_module == tmpLabelModule);
    
    matrix_connection_module(iSourceColumn, 1) = ...
        sum(matrix_connection_normalized_exclude(iSourceColumn, tmpIndexColumnInSameModule));
    
    matrix_connection_module(iSourceColumn, 2) = ...
        sum(matrix_connection_normalized_exclude(iSourceColumn, :)) - matrix_connection_module(iSourceColumn, 1);
end

matrix_connection_module(:,3) = matrix_connection_module(:,1)./matrix_connection_module(:,2);
        
    



% 
% counter_among = 0;
% counter_across = 0;
% 
% vector_strength_among = [];
% vector_strength_across = [];
%              
% for iColumn = 1:1:NUM_COLUMN
%     for jColumn = 1:1:NUM_COLUMN
%         tmpIndexi = find(Label_domain(:,1) == iColumn);
%         tmpIndexj = find(Label_domain(:,1) == jColumn);
%         
%         % exclude columns with neurons fewer than 20
%         if(isempty(tmpIndexi) || isempty(tmpIndexj))
%             continue;
%         end
%         
%         % self-connection
%         if(tmpIndexi == tmpIndexj)
%             continue;
%         end
%         
%         if(Label_domain(tmpIndexi, 2) == Label_domain(tmpIndexj,2))
%             counter_among = counter_among + 1;
%             vector_strength_among = [vector_strength_among, matrix_connection_normalized(iColumn, jColumn)];
%         else
%             counter_across = counter_across + 1;
%             vector_strength_across = [vector_strength_across, matrix_connection_normalized(iColumn, jColumn)];
%         end
%     end
% end
% 
% [H,P,CI,STATS] = ttest2(vector_strength_among, vector_strength_across, 'tail', 'right', 'vartype', 'unequal')
% 
% [P, H, STATS] = ranksum(vector_strength_among, vector_strength_across, 'alpha', 0.001, 'tail', 'right')
% 
% 
% vector_strength_value = [vector_strength_among, vector_strength_across];
% vector_strength_group = [repmat({'Among domain'}, 1, 290), repmat({'Across domain'}, 1, 640)];
% 
% boxplot(vector_strength_value, vector_strength_group)
% 
% 
% mean(vector_strength_among)
% mean(vector_strength_across)
% 
% 
% std(vector_strength_among)
% std(vector_strength_across)
% 
% 
% index_column_1 = Label_domain(find(Label_domain(:,2) == 1),1);
% index_column_2 = Label_domain(find(Label_domain(:,2) == 2),1);
% index_column_3 = Label_domain(find(Label_domain(:,2) == 3),1);
% 
% index_order = [index_column_1; index_column_2; index_column_3];
% 
% matrix_connection_redored = matrix_connection_normalized(:,index_order);
% matrix_connection_redored = matrix_connection_redored(index_order, :);
% 
% 
% figure;
% imagesc(matrix_connection_redored, [0,3000]);
% set(gca, 'XTick', 1:1:31, 'XTickLabel', index_order)
% set(gca, 'YTick', 1:1:31, 'YTickLabel', index_order)
% 

%% Visualization in Gephi
% Demo files

%node.csv
%Id,Label
%1,Cluster1
%2,Cluster2
%3,Cluster3
%4,Cluster4
%5,Cluster5

%edge.csv
%Source,Target,Weight,Type
%1,2,10,"Directed"
%1,3,100,"Directed"
%1,4,200,"Directed"
%1,5,50,"Directed"
%3,4,1,"Directed"
%4,3,100,"Directed"

fid_node = fopen('./Gephi/node.csv', 'w');
fprintf(fid_node, 'Id,Label,Domain\n');
for iNode = 1:1:NUM_COLUMN
    
    tmpIndex = find(Label_domain(:,1) == iNode);
    
    if(isempty(tmpIndex))
        continue;
    end
    fprintf(fid_node, '%d,%s,%d\n', iNode, num2str(iNode), Label_domain(tmpIndex, 2));
end
fclose(fid_node);


fid_edge = fopen('./Gephi/edge_distance.csv', 'w');
fprintf(fid_edge, 'Source,Target,Weight,Type\n');
for iSource = 1:1:NUM_COLUMN
    for iTarget = 1:1:NUM_COLUMN
        tmpIndexSource = find(Label_domain(:,1) == iSource);
        tmpIndexTarget = find(Label_domain(:,1) == iTarget);
        
        if(isempty(tmpIndexSource) || isempty(tmpIndexTarget))
            continue;
        end
        
        if(tmpIndexSource == tmpIndexTarget)
            continue;
        end
        
        
        fprintf(fid_edge, '%d,%d,%f,%s\n', ...
            iSource, iTarget, matrix_connection_normalized(iSource, iTarget), '"Directed"');
    end
end
fclose(fid_edge);



NUM_COLUMN = 31;
matrix_connection_normalized_exclude = matrix_connection_normalized_raw(Label_domain(:,1), :);
matrix_connection_normalized_exclude = matrix_connection_normalized_exclude(:, Label_domain(:,1));

matrix_connection_normalized_exclude_distance = ...
    matrix_connection_normalized_exclude.*(matrix_distance./max(matrix_distance(:)));


matrix_connection_normalized = matrix_connection_normalized_exclude;

% 
% upper = triu(matrix_connection_normalized);
% lower = tril(matrix_connection_normalized)';
% average = upper + lower;
% 
% matrix_connection_normalized = average + average';

parpool(8)

%% Louvain clustering 

%% shuffled matrix
shuffle_row = randperm(NUM_COLUMN);
shuffle_col = randperm(NUM_COLUMN);

matrix_connection_normalized_shuffled = matrix_connection_normalized(shuffle_row, :);
matrix_connection_normalized_shuffled = matrix_connection_normalized_shuffled(shuffle_col, :);

%
NUM_ITERATION = 1000;

gamma = (0.1:0.1:2.0)';
NUM_GAMMA = length(gamma);

matrix_M_shuffled = zeros(NUM_GAMMA, NUM_COLUMN, 'uint8');
vector_Q_shuffled = zeros(NUM_GAMMA, 2); % mean and std
vector_count_shuffled = zeros(NUM_GAMMA, 1);

parfor iGamma = 1:1:NUM_GAMMA
    tmpMatrixM = zeros(NUM_ITERATION, NUM_COLUMN);
    tmpVectorQ = zeros(NUM_ITERATION, 1);
    for iIteration = 1:1:NUM_ITERATION
        [M, Q] = community_louvain(matrix_connection_normalized_shuffled, gamma(iGamma,1));
        tmpMatrixM(iIteration, :) = M';
        tmpVectorQ(iIteration, 1) = Q;
    end
    
    % encode label vector as label string.
    tmpStringM = cell(NUM_ITERATION, 1);
    for iIteration = 1:1:NUM_ITERATION
        tmpString = '';
        for iNode = 1:1:NUM_COLUMN
            tmpString = strcat(tmpString, num2str(tmpMatrixM(iIteration, iNode), '%02d'));
        end
        tmpStringM{iIteration, 1} = tmpString;
    end
    
    % find the pattern of the most frequency
    tmpFreq = tabulate(tmpStringM);
    
    NUM_PATTERN = size(tmpFreq, 1);
    tmpCount = zeros(NUM_PATTERN, 1);
    for iPattern = 1:1:NUM_PATTERN
        tmpCount(iPattern, 1) = tmpFreq{iPattern, 2};
    end
    
    % TODO-what if there are more than one patterns found.
    tmpIndexMost = find(max(tmpCount(:)) == tmpCount);
    
    tmpPattern = tmpFreq{tmpIndexMost, 1};
    
    % vector_count
    vector_count_shuffled(iGamma, 1) = tmpFreq{tmpIndexMost, 2};
    
    tmpVectorQ2 = zeros(vector_count_shuffled(iGamma, 1), 1);
    tmpCounter = 0;
    for iIteration = 1:1:NUM_ITERATION
        tmpIndex = strfind(tmpStringM{iIteration, 1}, tmpPattern);
        if(~isempty(tmpIndex))
            tmpCounter = tmpCounter + 1;
            tmpVectorQ2(tmpCounter, 1) = tmpVectorQ(iIteration, 1);
        end
    end
    
    vector_Q_shuffled(iGamma, :) = [mean(tmpVectorQ2), std(tmpVectorQ2)];
    
    
    % decode label string to label vector
    tmpVectorM = zeros(NUM_COLUMN, 1);
    for iNode = 1:1:NUM_COLUMN
        tmpVectorM(iNode, 1) = str2num(tmpPattern(1, 2*iNode-1:2*iNode));
    end
    
    matrix_M_shuffled(iGamma, :) = tmpVectorM';
    
    disp(iGamma)
end


%% 
NUM_ITERATION = 1000;

gamma = (0.1:0.1:2.0)';
NUM_GAMMA = length(gamma);

matrix_M = zeros(NUM_GAMMA, NUM_COLUMN, 'uint8');
vector_Q = zeros(NUM_GAMMA, 2); % mean and std
vector_count = zeros(NUM_GAMMA, 1);

parfor iGamma = 1:1:NUM_GAMMA
    tmpMatrixM = zeros(NUM_ITERATION, NUM_COLUMN);
    tmpVectorQ = zeros(NUM_ITERATION, 1);
    for iIteration = 1:1:NUM_ITERATION
        [M, Q] = community_louvain(matrix_connection_normalized, gamma(iGamma,1));
        tmpMatrixM(iIteration, :) = M';
        tmpVectorQ(iIteration, 1) = Q;
    end
    
    % encode label vector as label string.
    tmpStringM = cell(NUM_ITERATION, 1);
    for iIteration = 1:1:NUM_ITERATION
        tmpString = '';
        for iNode = 1:1:NUM_COLUMN
            tmpString = strcat(tmpString, num2str(tmpMatrixM(iIteration, iNode), '%02d'));
        end
        tmpStringM{iIteration, 1} = tmpString;
    end
    
    % find the pattern of the most frequency
    tmpFreq = tabulate(tmpStringM);
    
    NUM_PATTERN = size(tmpFreq, 1);
    tmpCount = zeros(NUM_PATTERN, 1);
    for iPattern = 1:1:NUM_PATTERN
        tmpCount(iPattern, 1) = tmpFreq{iPattern, 2};
    end
    
    % TODO-what if there are more than one patterns found.
    tmpIndexMost = find(max(tmpCount(:)) == tmpCount);
    
    tmpPattern = tmpFreq{tmpIndexMost, 1};
    
    % vector_count
    vector_count(iGamma, 1) = tmpFreq{tmpIndexMost, 2};
    
    tmpVectorQ2 = zeros(vector_count(iGamma, 1), 1);
    tmpCounter = 0;
    for iIteration = 1:1:NUM_ITERATION
        tmpIndex = strfind(tmpStringM{iIteration, 1}, tmpPattern);
        if(~isempty(tmpIndex))
            tmpCounter = tmpCounter + 1;
            tmpVectorQ2(tmpCounter, 1) = tmpVectorQ(iIteration, 1);
        end
    end
    
    vector_Q(iGamma, :) = [mean(tmpVectorQ2), std(tmpVectorQ2)];
    
    
    % decode label string to label vector
    tmpVectorM = zeros(NUM_COLUMN, 1);
    for iNode = 1:1:NUM_COLUMN
        tmpVectorM(iNode, 1) = str2num(tmpPattern(1, 2*iNode-1:2*iNode));
    end
    
    matrix_M(iGamma, :) = tmpVectorM';
    
    disp(iGamma)
end

%% Determine number of modules (reference: 2019, Nature, AIBS)
figure;
plot(gamma, [vector_Q_shuffled(:,1), vector_Q(:,1)])
legend('Q_{shuffled}', 'Q')

diff_Q = vector_Q(:,1) - vector_Q_shuffled(:,1);
gamma(diff_Q == max(diff_Q), 1)

%% generate ordered heatmap based on community
for iLevel = 10%7:1:13

%level = 7;
% 
[Y, I] = sort(matrix_M(iLevel, :));

matrix_row = matrix_connection_normalized_exclude(I,:);

matrix_ordered = matrix_row(:,I);

figure;
imagesc(matrix_ordered, [0, 2000]);

set(gca, 'XTick', 1:1:NUM_COLUMN, 'XTickLabel', Label_domain(I,1))
set(gca, 'YTick', 1:1:NUM_COLUMN, 'YTickLabel', Label_domain(I,1))
axis equal



% Write community to tif
community_PFC = mask;
for iNode = 1:1:NUM_COLUMN
    community_PFC(mask == iNode) = matrix_M(iLevel, iNode);
end

func_matrix2tiff(community_PFC, 8, strcat('./louvain_community2/community_PFC4_gamma_',num2str(gamma(iLevel,1), '%0.1f'),'.tif'));

end


%% Calculate column-wise distance for regression
anno_CC = nrrdread('../IntraPFC_connection/result/label_PFC_volume_filled_25um.nrrd');
resolution_CC = 25;

matrix_centriod = zeros(40, 3);

for iColumn = 1:1:40
    [tmpRow, tmpCol, tmpSlice] = ind2sub([320, 528, 456], find(anno_CC == iColumn));
    
    matrix_centriod(iColumn, :) = [mean(tmpRow), mean(tmpCol), mean(tmpSlice)];
    
end

matrix_distance = zeros(40, 40);

for iSource = 1:1:40
    for iTarget = 1:1:40
        matrix_distance(iSource, iTarget) = sqrt(sum((matrix_centriod(iSource,:) - matrix_centriod(iTarget,:)).^2));
    end
end
        
matrix_distance = matrix_distance(Label_domain(:,1), :);
matrix_distance = matrix_distance(:, Label_domain(:,1));

%% export matrix for Gephi community
label_domain_gephi=[6,11,12,14,18,28,30,34,39,40,2,10,20,23,24,32,33,36,37,38,1,7,9,13,15,16,19,22,25,26,35];
matrix_gephi = matrix_connection_normalized(label_domain_gephi,:);
matrix_gephi = matrix_gephi(:,label_domain_gephi);
figure;
imagesc(matrix_gephi, [0,4000])

set(gca, 'XTick', 1:1:NUM_COLUMN, 'XTickLabel', label_domain_gephi);
set(gca, 'YTick', 1:1:NUM_COLUMN, 'YTickLabel', label_domain_gephi);
axis equal

%%
