# PT and CT neurons MD
#
# 2020/6/26-GAO Le


library("gplots")
library("fastcluster")
library("ComplexHeatmap")
library("dendextend")
library("colorspace")
library("amap")

source('~/Documents/YanLab/Task_Local/shared_R/public.r')

setwd("~/Documents/YanLab/Task_Local/BrainRegionSubdivision/MD_PT_CT_v2/")


proj = read.table("./projection_normalization_no.txt")
proj_name_neuron = read.table("./name_neuron.txt")

proj_as_matrix = as.matrix(proj)
rownames(proj_as_matrix) <- as.character(proj_name_neuron[,1])

NumNeuron = dim(proj_as_matrix)[1]
NumRegion = dim(proj_as_matrix)[2]

LabelRegion = func_getLabelRegion(NumNeuron = NumNeuron, proj_name_neuron = proj_name_neuron)
LabelSubtype = func_getLabelSubtype(NumNeuron = NumNeuron, proj_name_neuron = proj_name_neuron)
NeuronDistribution = func_getRegionDistribution(proj_name_neuron[,1])

SWCPath = func_getSWCPath(proj_name_neuron)

##

LabelSourceRegion_Type = paste0(LabelRegion, LabelSubtype)


##
row_ha = HeatmapAnnotation(region = LabelRegion, 
                           col = list(region = ColorSourceRegion), 
                           which = "row")

library("circlize")
col_fun = colorRamp2(c(0,60), c("white", "black"))


# Complex Heatmap.
ht = Heatmap(matrix = proj_as_matrix, 
             name = "Axon length (um)",
             col = col_fun,
             clustering_distance_columns = "spearman", 
             clustering_distance_rows = "spearman",
             clustering_method_columns = "ward.D2",
             clustering_method_rows = "ward.D2",
             
             show_row_names = FALSE,
             show_column_names = FALSE,
             left_annotation = row_ha,
             
             #cluster_rows = rowDend,
             #cluster_columns = columnDend,
             
             #cluster_columns = columnDend,
             cluster_rows = TRUE,
             
             show_row_dend = FALSE,

             #column_split = 5,
             
             
             row_dend_side = "left",
             
             width = unit(6, "cm"), 
             height = unit(4, "cm"),
             column_names_rot = 45,
             column_names_gp = gpar(fontsize = 10),
             use_raster = TRUE,
             border = TRUE
             )
draw(ht)


##
columnDend = column_dend(ht)
columnOrder = column_order(ht)



##
NumCluster = 5

label_cluster_cube = cutree(tree = columnDend, k = NumCluster)
label_on_columnDend = unique(label_cluster_cube[columnOrder])


proj_domain = array(data = 0, dim = c(NumNeuron, NumCluster))
volume_domain = array(data = 0, dim = c(NumCluster,1))
for(iDomain in 1:NumCluster) {
  volume_domain[iDomain, 1] = length(which(label_cluster_cube == label_on_columnDend[iDomain]))
  proj_domain[,iDomain] = 
    rowSums(proj_as_matrix[,which(label_cluster_cube == label_on_columnDend[iDomain])])/ volume_domain[iDomain, 1]
}


proj_domain_source = array(data = 0, dim = c(11, NumCluster))
for(iSourceRegion in 1:11) {
  proj_domain_source[iSourceRegion,] = 
    colMedians(proj_domain[which(SourceRegion[iSourceRegion] == LabelRegion),])
  
  print(SourceRegion[iSourceRegion])
  print(length(which(SourceRegion[iSourceRegion] == LabelRegion)))
}


rownames(proj_domain_source) = SourceRegion
colnames(proj_domain_source) = seq(1,NumCluster,1)

proj_domain_source_normalized = apply(proj_domain_source, 2, function(x) (x-min(x))/(max(x)-min(x)))



Heatmap(matrix = t(proj_domain_source_normalized),
        # cluster_columns = FALSE,
        # cluster_rows = FALSE,
        clustering_distance_columns = "spearman",
        clustering_method_columns = "ward.D2",
        clustering_distance_rows = "spearman",
        clustering_method_rows = "ward.D2")


## flatmap CT+PT
label_domain_neuron = array(data = 0, dim = c(NumNeuron, 1))
for(iNeuron in 1:NumNeuron) {
  label_domain_neuron[iNeuron, 1] = which(max(proj_domain[iNeuron,]) == proj_domain[iNeuron,])
}


write.table(label_domain_neuron, "./label_domain_neuron.txt")

## flatmap CT

write.table(LabelSubtype, "./label_domain_neuron_subtype.txt", col.names = F)





## 

proj_source_CT = array(data = 0, dim = c(NumSourceRegion, 5))

for(iSourceRegion in 1:NumSourceRegion) {
  
  proj_source_CT[iSourceRegion, ] = colMedians(proj_domain[base::intersect(which(LabelRegion == SourceRegion[iSourceRegion]),
                                                            which(LabelSubtype == "CT")), ])
}

rownames(proj_source_CT) = paste0(SourceRegion, "+CT")


proj_source_PT = array(data = 0, dim = c(NumSourceRegion, 5))

for(iSourceRegion in 1:NumSourceRegion) {
  
  proj_source_PT[iSourceRegion, ] = colMedians(proj_domain[base::intersect(which(LabelRegion == SourceRegion[iSourceRegion]),
                                                                     which(LabelSubtype == "PT")), ])
}
rownames(proj_source_PT) = paste0(SourceRegion, "+PT")



proj_source_CT_total = array(data = 0, dim = c(NumSourceRegion, 6)) # 5+1 
proj_source_CT_total[,1:5] = proj_source_CT
proj_source_CT_total[,6] = rowSums(proj_source_CT)

proj_source_PT_total = array(data = 0, dim = c(NumSourceRegion, 6)) # 5+1 
proj_source_PT_total[,1:5] = proj_source_PT
proj_source_PT_total[,6] = rowSums(proj_source_PT)


proj_source_subtype = cbind(proj_source_CT, proj_source_PT)
#colnames(proj_source_subtype) = seq(1,5,1)

Heatmap(proj_source_subtype,
        col = colorRamp2(c(0,1,4), c("blue", "white", "red")),
        clustering_distance_rows = "spearman",
        clustering_method_rows = "ward.D2",
        cluster_rows = TRUE,
        cluster_columns = FALSE,
        column_split = c(replicate(5, "CT"), replicate(5, "PT")),
        
        row_names_side = "left"
        
        )

df_sutype = data.frame(CT = proj_source_CT_total[,6], PT = proj_source_PT_total[,6])

ggpaired(df_sutype, cond1 = "CT", cond2 = "PT", color = "condition", 
         filled = "condition", palette = c("red", "green")) +
  stat_compare_means()


sta_consistent = array(data = 0, dim = c(NumSourceRegion, NumSourceRegion))

sta_within_region = array(data = 0, dim = c(11,1))
sta_without_region = array(data = 0, dim = c(11*11-11,1))

counter_within = 0
counter_without = 0

for(iSourceRegionCT in 1:NumSourceRegion) {
  for(iSourceRegionPT in 1:NumSourceRegion) {
    sta_consistent[iSourceRegionCT, iSourceRegionPT] = 
      cor(proj_source_CT_total[iSourceRegionCT,1:5], proj_source_PT_total[iSourceRegionPT,1:5], 
        method = "spearman")
    
    if(iSourceRegionCT == iSourceRegionPT) {
      counter_within = counter_within  + 1
      sta_within_region[counter_within, 1] = sta_consistent[iSourceRegionCT, iSourceRegionPT]
    }
    else {
      counter_without = counter_without + 1
      sta_without_region[counter_without, 1] = sta_consistent[iSourceRegionCT, iSourceRegionPT]
    }
      
  }
}


rownames(sta_consistent) = SourceRegion
colnames(sta_consistent) = SourceRegion

ggcorrplot(sta_consistent, lab = F, type = "full")


df_within_without = data.frame(value = rbind(sta_within_region, sta_without_region),
                              group = c(replicate(length(sta_within_region), "Within"), 
                                       replicate(length(sta_without_region), "Without")))

ggboxplot(data = df_within_without, x = "group", y = "value") +
  stat_compare_means()
