##################
###load packages
library(nat)
library(WGCNA)
library(ggplot2)
library(ggridges)
library(ggpubr)
library(circlize)

###load data and functions
DATADIR<-"~/课题/Work\ report/Code/Data/"
FDIR<-"~/课题/Work\ report/Code/ForFigures/"
FIGUREDIR<-"~/课题/Work\ report/Code/Figures/"
if(!dir.exists(FIGUREDIR)){
  dir.create(FIGUREDIR)
}

load(paste0(DATADIR,"Neurons"))
load(paste0(DATADIR,"Trunks"))
load(paste0(DATADIR,"Segments_PT"))
load(paste0(DATADIR,"Segmentsdistmatx_PT"))
load(paste0(DATADIR,"Collateral_PT"))

source(paste0(FDIR,"ForFigures.R"))
##################
####Figure 4b,c
##align the primary axons
#estimate planes according to the orietation of primary axons
Paxons<-subset(Trunks,Type=="PT")
somapos<-data.frame(X=unlist(lapply(Paxons,function(x)x$d$X[1])),Y=unlist(lapply(Paxons,function(x)x$d$Y[1])),Z=unlist(lapply(Paxons,function(x)x$d$Z[1])))
Paxons[,]$Z<-somapos$Z

maxid<-c()
for(i in 1:length(Paxons)){
  maxid<-c(maxid,length(Paxons[[i]]$d$PointNo))
}

cp<-colSums(somapos)/length(Paxons)#initial center position
m<-50#step length

# initial vector
v<-c(0,0,0)
for(i in 1:length(Paxons)){
  v<-v+Paxons[[i]]$d[1+m,3:5]-Paxons[[i]]$d[1,3:5]
}
v<-v/length(Paxons)

alignmentid<-rep(1,length(Paxons))#initial alignment
Alignmentid<-c()
CP<-c()#origin of the planes
V<-c()
n<-1#plane index
repeat{
  
  if(all(alignmentid==maxid)){break}#test whether planes move to ends of all axons
  
  print(n)
  n<-n+1
  
  CP<-rbind(CP,cp)
  V<-rbind(V,v)
  
  #plane function
  A<-as.numeric(v[1]/sqrt(sum(v^2)))
  B<-as.numeric(v[2]/sqrt(sum(v^2)))
  C<-as.numeric(v[3]/sqrt(sum(v^2)))
  D<-as.numeric(-sum(v*cp)/sqrt(sum(v^2)))
  
  #renew v and cp according to points in the plane
  v<-c(0,0,0)
  cp<-c(0,0,0)
  for(i in 1:length(Paxons)){
    repeat{
      disttoplane<-(A*Paxons[[i]]$d$X[alignmentid[i]]+
                     B*Paxons[[i]]$d$Y[alignmentid[i]]+
                     C*Paxons[[i]]$d$Z[alignmentid[i]]+D)/sqrt(A^2+B^2+C^2)
      if(disttoplane>0|alignmentid[i]==maxid[i]){break}else{
        alignmentid[i]<-alignmentid[i]+1
      }
    }
    
    if(alignmentid[i]!=1&alignmentid[i]!=maxid[i]){
      v<-v+Paxons[[i]]$d[min(alignmentid[i]+m,maxid[i]),3:5]-Paxons[[i]]$d[alignmentid[i],3:5]
      cp<-cp+Paxons[[i]]$d[alignmentid[i],3:5]-disttoplane*c(A,B,C)
    }
  }
  v<-v/length(which(alignmentid!=1&alignmentid!=maxid))
  cp<-cp/length(which(alignmentid!=1&alignmentid!=maxid))+v
  
  Alignmentid<-rbind(Alignmentid,alignmentid)
}
Alignmentid[1,]<-2
CP<-CP-V
CP[1:nrow(CP)-1,]<-CP[2:nrow(CP),]

#number of axons in each cross-sections
naxons=c()
for(i in 1:nrow(Alignmentid)){
  naxons=c(naxons,length(which(Alignmentid[i,]!=1&Alignmentid[i,]!=maxid )))
}
sPlanes<-1:length(naxons)

##calculate the mean distance from points in each plane to the soma
Mean_disttosoma<-rep(NA,length(sPlanes))
for(n in sPlanes){
  print(n)
  disttosoma<-rep(NA,length(which(Alignmentid[n,]!=1&Alignmentid[n,]!=maxid)))
  k<-1
  for(i in which(Alignmentid[n,]!=1&Alignmentid[n,]!=maxid)){
    disttosoma[k]<-sum(sqrt(rowSums((Paxons[[i]]$d[2:Alignmentid[n,i],3:5]-Paxons[[i]]$d[1:(Alignmentid[n,i]-1),3:5])^2)))
    k<-k+1
  }
  Mean_disttosoma[which(sPlanes==n)]<-mean(disttosoma)
}

##calculate the points' position in the plane's coordination
Disttocenter<-matrix(NA,nrow(Alignmentid)-1,ncol(Alignmentid))#the dist between points to the origin in planes
Pointspos_nr<-vector("list",nrow(Alignmentid)-1)#Points position in section without rotation
Vx<-c()#local coordination
Vy<-c()#local coordination
Dim_n<-c(1,0,0)#align the normal vector of the plane to Dim_n
Dim_x<-c(0,0,1)
Dim_y<-c(0,-1,0)
for(i in 1:(nrow(Alignmentid)-1)){
  print(i)
  
  #plane function
  A<-as.numeric(V[i,1]/sqrt(sum(V[i,]^2)))
  B<-as.numeric(V[i,2]/sqrt(sum(V[i,]^2)))
  C<-as.numeric(V[i,3]/sqrt(sum(V[i,]^2)))
  D<-as.numeric(-sum(V[i,]*CP[i,])/sqrt(sum(V[i,]^2)))
  
  #translation 
  Tmatx<-as.matrix(rbind(c(1,0,0,0),c(0,1,0,0),c(0,0,1,0),cbind(-CP[i,],1)))#transform coordination
  
  #rotate transformation along v3
  v2<-matrix(Dim_n,3,1)
  v1<-matrix(unlist(V[i,]/sqrt(sum(V[i,]^2))),3,1)#the normal vector of the plane
  theta<-acos(t(v1) %*% v2/sqrt(sum(v1^2)*sum(v2^2)))#the angle between v1 and v2
  v3<-rbind(v1[2]*v2[3]-v1[3]*v2[2],v1[3]*v2[1]-v1[1]*v2[3],v1[1]*v2[2]-v1[2]*v2[1])#the vector normal to v1 and v2
  v3<-v3/sqrt(sum(v3^2))
  Rmatx<-as.matrix(rbind(c(v3[1]^2+(1-v3[1]^2)*cos(theta),v3[1]*v3[2]*(1-cos(theta))+v3[3]*sin(theta),v3[1]*v3[3]*(1-cos(theta))-v3[2]*sin(theta),0),
                        c(v3[1]*v3[2]*(1-cos(theta))-v3[3]*sin(theta),v3[2]^2+(1-v3[2]^2)*cos(theta),v3[2]*v3[3]*(1-cos(theta))+v3[1]*sin(theta),0),
                        c(v3[1]*v3[3]*(1-cos(theta))+v3[2]*sin(theta),v3[2]*v3[3]*(1-cos(theta))-v3[1]*sin(theta),v3[3]^2+(1-v3[3]^2)*cos(theta),0),
                        c(0,0,0,1)))#rotate matrix
  
  #calculate the local coordination
  Vx<-rbind(Vx,t(matrix(c(Dim_x,1),1,4) %*% solve(Rmatx))[1:3])
  Vy<-rbind(Vy,t(matrix(c(Dim_y,1),1,4) %*% solve(Rmatx))[1:3])
  
  #calculate points in the plane coordination
  pointspos<-matrix(NA,ncol(Alignmentid),2)  
  for(j in 1:ncol(Alignmentid)){
    disttoplane<-(A*Paxons[[j]]$d$X[Alignmentid[i,j]]+
                   B*Paxons[[j]]$d$Y[Alignmentid[i,j]]+
                   C*Paxons[[j]]$d$Z[Alignmentid[i,j]]+D)/sqrt(A^2+B^2+C^2)
    P<-Paxons[[j]]$d[Alignmentid[i,j],3:5]-disttoplane*c(A,B,C)
    Disttocenter[i,j]<-dist(rbind(P,CP[i,]))
    
    tmp<-cbind(P$X,P$Y,P$Z,1) %*% Tmatx %*% Rmatx
    pointspos[j,]<-c(tmp[1:3] %*% matrix(Dim_x,3,1),tmp[1:3] %*% matrix(Dim_y,3,1))
    
  }
  Pointspos_nr[[i]]<-pointspos
}

##select trunks
Starti<-15#start plane
Endi<-65#end plane
Neuronid<-1:length(Alignmentid[1,])
for(i in Starti:Endi){
  Neuronid<-intersect(Neuronid,which(Alignmentid[i,]!=1&Alignmentid[i,]!=maxid))
}

###Figure 4b
open3d()
plot3d(CP[sPlanes,],size = 5,col="black",add=T)
Color<-read.csv(paste0(FDIR,"color.txt"),sep=",",header = F)
subtrunks<-subset(Paxons,Neuronid)
plot3d(subtrunks,soma=30,col=as.character(Color$V1[match(subtrunks[,]$BrainRegion,as.character(Color$V2))]),add=T)
lines3d(CP$X[sPlanes],CP$Y[sPlanes],CP$Z[sPlanes],col="red")

i=Starti#plane index
A=as.numeric(V[i,1]/sqrt(sum(V[i,]^2)))
B=as.numeric(V[i,2]/sqrt(sum(V[i,]^2)))
C=as.numeric(V[i,3]/sqrt(sum(V[i,]^2)))
D=as.numeric(-sum(V[i,]*CP[i,])/sqrt(sum(V[i,]^2)))
planes3d(A,B,C,D,col="red",alpha=0.1)
arrow3d(p0=CP[i,],p1=CP[i,]+3000*Vx[i,],type="lines",col="blue",n=2,s=1/10)
arrow3d(p0=CP[i,],p1=CP[i,]-3000*Vx[i,],type="lines",col="blue",s=0)
arrow3d(p0=CP[i,],p1=CP[i,]+3000*Vy[i,],type="lines",n=2,s=1/10)
arrow3d(p0=CP[i,],p1=CP[i,]-3000*Vy[i,],type="lines",s=0)
arrow3d(p0=CP[i,],p1=CP[i,]+3000*V[i,]/sqrt(sum(V[i,]^2)),type="lines",col="red",n=2,s=1/10)

i=Endi#plane index
A=as.numeric(V[i,1]/sqrt(sum(V[i,]^2)))
B=as.numeric(V[i,2]/sqrt(sum(V[i,]^2)))
C=as.numeric(V[i,3]/sqrt(sum(V[i,]^2)))
D=as.numeric(-sum(V[i,]*CP[i,])/sqrt(sum(V[i,]^2)))
planes3d(A,B,C,D,col="red",alpha=0.1)
arrow3d(p0=CP[i,],p1=CP[i,]+3000*Vx[i,],type="lines",col="blue",n=2,s=1/10)
arrow3d(p0=CP[i,],p1=CP[i,]-3000*Vx[i,],type="lines",col="blue",s=0)
arrow3d(p0=CP[i,],p1=CP[i,]+3000*Vy[i,],type="lines",n=2,s=1/10)
arrow3d(p0=CP[i,],p1=CP[i,]-3000*Vy[i,],type="lines",s=0)
arrow3d(p0=CP[i,],p1=CP[i,]+3000*V[i,]/sqrt(sum(V[i,]^2)),type="lines",col="red",n=2,s=1/10)

umatx<-matrix(c(0.97,-0.07,0.24,0,-0.01,-0.97,-0.24,0,0.26,0.22,-0.94,0,0,0,0,1),4,4)
par3d(windowRect = c(103,202,1396,1051),zoom=.7,userMatrix=umatx)

ticklen<-2000
ticknum<-2
tickcex<-0.7
textcex<-1
textline<-0.2
xtick<-seq(2150,by=ticklen,length.out = ticknum)
ytick<-seq(1048,by=ticklen,length.out = ticknum)
ztick<-seq(1434,by=ticklen,length.out = ticknum)
axis3d(edge="x",at=xtick,tick=F,labels =F,cex=tickcex)
axis3d(edge="y",at=ytick,tick=F,labels =F,cex=tickcex)
axis3d(edge="z",at=ztick,tick=F,labels =F,cex=tickcex)
mtext3d(text="A->P", edge="x", at = xtick[1]+ticklen/2,line=textline,cex=textcex)
mtext3d(text="D->V", edge="y", at = ytick[1]+ticklen/2,line=textline,cex=textcex)
mtext3d(text="L->M", edge="z", at = ztick[1]+ticklen/2,line=textline,cex=textcex)

rgl.snapshot(paste0(FIGUREDIR,"Figure 4b.png"))
rgl.close()

##inset
i<-Starti
orders<-order(unlist(lapply(subtrunks,function(x)x$d$Z[1])),decreasing = T)
pdf(paste0(FIGUREDIR,"Figure 4b(inset 1).pdf"),width=4,height = 4)
plot(Pointspos_nr[[i]][Neuronid,1][orders],Pointspos_nr[[i]][Neuronid,2][orders],col=as.character(Color$V1[match(subtrunks[,]$BrainRegion,as.character(Color$V2))])[orders],pch=20,cex=0.5,asp = 1,xlab="",ylab="",xaxt="n",yaxt="n")
points(0,0,lwd=2,cex=1,pch=21,bg="white")
arrows(0,0,2500,0,lwd=2,length=0.1)
arrows(0,0,0,2500,lwd=2,length=0.1)
text(c(2800,0),c(0,2800),c("X1","Y1"))
legend("topright","Section 1",text.col="red",bty="n")
dev.off()

i<-Endi
orders<-order(unlist(lapply(subtrunks,function(x)x$d$Z[1])),decreasing = T)
pdf(paste0(FIGUREDIR,"Figure 4b(inset 2).pdf"),width=4,height = 4)
plot(Pointspos_nr[[i]][Neuronid,1][orders],Pointspos_nr[[i]][Neuronid,2][orders],col=as.character(Color$V1[match(subtrunks[,]$BrainRegion,as.character(Color$V2))])[orders],pch=20,cex=0.5,asp = 1,xlab="",ylab="",xaxt="n",yaxt="n")
points(0,0,lwd=2,cex=1,pch=21,bg="white")
arrows(0,0,1500,0,lwd=2,length=0.1)
arrows(0,0,0,1500,lwd=2,length=0.1)
text(c(1700,0),c(0,1700),c("X2","Y2"))
legend("topright","Section 2",text.col="red",bty="n")
dev.off()

###Figure 4c
cori<-rep(NA,Endi-Starti+1)
p<-rep(NA,Endi-Starti+1)
for(i in Starti:Endi){
  print(i)
  fspace<-somapos[Neuronid,]
  pointspos<-matrix(NA,nrow=length(Neuronid),ncol=3)
  for(n in Neuronid){
    pointspos[which(Neuronid==n),]<-as.matrix(Paxons[[n]]$d[Alignmentid[i,n],3:5])
  }
  dist_f<-dist(fspace)
  dist_m<-dist(pointspos)
  tmp<-cor.test(as.vector(dist_f),as.vector(dist_m))
  cori[i-Starti+1]<-tmp$estimate
  p[i-Starti+1]<-tmp$p.value
}

pc<-rep(1,length(cori))
pc[which(p<0.05)]<-16
pdf(paste0(FIGUREDIR,"Figure 1c(1).pdf"),width=4,height = 4)
par(mar=c(5.1,8.1,4.1,2.1))
plot(Mean_disttosoma[Starti:Endi]/1000,cori,pch=pc,type="b",xlab="Mean distance to somata (mm)",ylab="Correlation between positions of \nintersection points and somata",ylim=c(0,1),col="red",cex=0.5)
dev.off()

MeanRotAng<-c()
VP<-c()
VP_3d<-c()
gradient<-"Y"
for(i in 1:(nrow(Alignmentid)-1)){
  #calculate the mean twist angle by multiple linear fit
  fit<-lm(somapos[which(!is.na(Pointspos_nr[[Starti]][,1])),gradient]~Pointspos_nr[[Starti]][which(!is.na(Pointspos_nr[[Starti]][,1])),1]+Pointspos_nr[[Starti]][which(!is.na(Pointspos_nr[[Starti]][,1])),2])
  v2<-fit$coefficients[2:3]
  
  if(naxons[i]>3){
    # v2=c(1,0)
    fit<-lm(somapos[which(!is.na(Pointspos_nr[[i]][,1])),gradient]~Pointspos_nr[[i]][which(!is.na(Pointspos_nr[[i]][,1])),1]+Pointspos_nr[[i]][which(!is.na(Pointspos_nr[[i]][,1])),2])
    v1=fit$coefficients[2:3]
    VP<-rbind(VP,v1/sqrt(sum(v1^2)))
    a=acos(v1 %*% v2/sqrt(sum(v1^2)*sum(v2^2)))
    if(is.na(a)){
      a<-0
    }
    if((v1[1]*v2[2]-v1[2]*v2[1])<0&a/pi*180<=90){
      MeanRotAng=c(MeanRotAng,-a/pi*180)
    }else if((v1[1]*v2[2]-v1[2]*v2[1])<0&a/pi*180>90){
      MeanRotAng=c(MeanRotAng,-a/pi*180)
    }else{
      MeanRotAng=c(MeanRotAng,a/pi*180)
    }
    
    ##calculate VP_3d
    vp<-v1/sqrt(sum(v1^2))
    v1=matrix(Dim_n,3,1)#x axis
    v2=matrix(unlist(V[i,]/sqrt(sum(V[i,]^2))),3,1)#the normal vector of the plane
    theta=acos(t(v1) %*% v2/sqrt(sum(v1^2)*sum(v2^2)))#the angle between v1 and v2
    v3=rbind(v1[2]*v2[3]-v1[3]*v2[2],v1[3]*v2[1]-v1[1]*v2[3],v1[1]*v2[2]-v1[2]*v2[1])#the vector normal to v1 and v2
    v3=v3/sqrt(sum(v3^2))
    Rmatx=as.matrix(rbind(c(v3[1]^2+(1-v3[1]^2)*cos(theta),v3[1]*v3[2]*(1-cos(theta))+v3[3]*sin(theta),v3[1]*v3[3]*(1-cos(theta))-v3[2]*sin(theta),0),
                          c(v3[1]*v3[2]*(1-cos(theta))-v3[3]*sin(theta),v3[2]^2+(1-v3[2]^2)*cos(theta),v3[2]*v3[3]*(1-cos(theta))+v3[1]*sin(theta),0),
                          c(v3[1]*v3[3]*(1-cos(theta))+v3[2]*sin(theta),v3[2]*v3[3]*(1-cos(theta))-v3[1]*sin(theta),v3[3]^2+(1-v3[3]^2)*cos(theta),0),
                          c(0,0,0,1)))#rotate matrix
    tmp=c((matrix(c(0,vp),1,3) %*% rbind(Dim_n,Dim_x,Dim_y)),1) %*% Rmatx
    VP_3d<-rbind(VP_3d,tmp[1:3])
  }else{
    MeanRotAng=c(MeanRotAng,NA)
  }
}

pdf(paste0(FIGUREDIR,"Figure 1c(2).pdf"),width=4,height = 4)
par(mar=c(5.1,8.1,4.1,2.1))
plot(Mean_disttosoma[Starti:Endi]/1000,MeanRotAng[Starti:Endi],pch=16,type="b",xlab="Mean distance to somata (mm)",ylab ="Mean rotational angle (Degree)",col="red",cex=0.5)
dev.off()

##################
####Figure 4d
out.hclust<-hclust(as.dist(distmatx),method = "ward.D2")
m<-cutreeHybrid(out.hclust,distmatx,minClusterSize = 3,deepSplit = 2.1)
Segments[,]$Subclass<-m$labels[match(names(Segments),rownames(distmatx))]
Segments[,]$Color<-getcol(Segments[,]$Subclass)

##ordered collateral clusters by path distance to soma
Collateral$CSubclass<-Segments[,]$Subclass[match(paste0(Collateral$NeuronName,"_",Collateral$BranchPointsId),paste0(Segments[,]$NeuronName,"_",Segments[,]$BranchPointsId))]
Collateral$Color<-Segments[,]$Color[match(paste0(Collateral$NeuronName,"_",Collateral$BranchPointsId),paste0(Segments[,]$NeuronName,"_",Segments[,]$BranchPointsId))]
meanpds<-rep(NA,length(unique(Collateral$CSubclass[which(!is.na(Collateral$CSubclass))])))
for(i in unique(Collateral$CSubclass[which(!is.na(Collateral$CSubclass))])){
  meanpds[which(unique(Collateral$CSubclass[which(!is.na(Collateral$CSubclass))])==i)]<-mean(Collateral$PathDisttoSoma[which(Collateral$CSubclass==i)])
}
Segments[,]$Subclass<-match(Segments[,]$Subclass,unique(Collateral$CSubclass[which(!is.na(Collateral$CSubclass))])[order(meanpds)])
Collateral$CSubclass<-Segments[,]$Subclass[match(paste0(Collateral$NeuronName,"_",Collateral$BranchPointsId),paste0(Segments[,]$NeuronName,"_",Segments[,]$BranchPointsId))]
Collateral$CSubclass<-factor(Collateral$CSubclass)

open3d()
plot3d(Segments,col=Segments[,]$Color,add=T)
shade3d(readOBJ(paste0(FDIR,"root.obj")),color="grey",override = T,alpha=0.05)
par3d(windowRect = c(103,202,1396,1051),zoom=0.5,userMatrix=matrix(c(1,0,0,0,0,-1,0,0,0,0,-1,0,0,0,0,1),4,4))
rgl.snapshot(paste0(FIGUREDIR,"Figure 4d.png"))
rgl.close()

##################
####Figure 4e
tmp<-subset(Collateral,!is.na(Collateral$CSubclass))
tmp$CSubclass<-as.numeric(tmp$CSubclass)
tmp$CSubclass<-factor(tmp$CSubclass,levels=max(tmp$CSubclass):1)
tmp$PathDisttoSoma<-tmp$PathDisttoSoma/1000
ggplot(tmp, aes(x =PathDisttoSoma, y=CSubclass,fill=CSubclass,height=..scaled..))+
  geom_density_ridges(stat = "density",alpha = .7,size=0.25)+
  labs(x="Distance to soma (mm)",y="Collateral clusters",title="Distribution of branch points \non the primary axon")+
  theme_classic()+
  theme(plot.title = element_text(hjust = 0.5,size=8),
        axis.text.x =element_text(size=7),axis.text.y =element_text(size=8),
        axis.title.x =element_text(size=8),axis.title.y =element_text(size=8),
        legend.position = "none")+
  scale_fill_manual(values=unique(tmp$Color)[match(levels(tmp$CSubclass),unique(tmp$CSubclass))])
ggsave(paste0(FIGUREDIR,"Figure 4e.pdf"),device = "pdf",width = 40,height = 50,scale=1,units = "mm")

##################
####Figure 4f,g
cb<-subset(Collateral,!is.na(Collateral$CSubclass))

###Figure 4f
cfreq<-rep(0,length(unique(cb$TargetRegion))*length(unique(cb$CSubclass)))
i<-1
for(s in unique(cb$TargetRegion)){
  for(cs in unique(cb$CSubclass)){
    cfreq[i]<-length(which(cb$TargetRegion==s&cb[,]$CSubclass==cs))/length(which(cb$CSubclass==cs))
    i<-i+1
  }
}

Data<-data.frame(Proportion=cfreq,TargetRegion=as.factor(rep(unique(cb[,]$TargetRegion),each=length(unique(cb[,]$CSubclass)))),CSubclass=factor(rep(unique(cb[,]$CSubclass),length(unique(cb[,]$TargetRegion)))))
ggplot(Data, aes(x =TargetRegion , y = Proportion,fill=CSubclass)) +
  geom_bar(stat = "identity", position = position_dodge(0.9))+
  facet_grid(CSubclass~.)+theme_classic()+
  labs(y="Proportion",title="Target regions of collateral clusters")+
  theme(plot.title = element_text(hjust = 0.5,size=8),
        axis.text.x = element_text(vjust = 0.5,angle = 90,size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_blank(),axis.title.y =element_text(size=8),
        strip.background = element_blank(),strip.text.y = element_text(angle=0,size=8),
        legend.position = "none")+
  scale_y_continuous(n.breaks = 2)+
  scale_fill_manual(values=unique(cb$Color)[match(levels(cb$CSubclass),unique(cb$CSubclass))])
ggsave(paste0(FIGUREDIR,"Figure 4f.pdf"),device = "pdf",width = 50,height = 50,scale=1,units = "mm")

###Figure 4g
psedo_bts<-rep(NA,nrow(cb))
dist2s<-rep(NA,nrow(cb))
for(i in 1:nrow(cb)){
  print(i)
  trunk<-Paxons[[which(names(Paxons)==cb$NeuronName[i])]]
  dist2tmc<-sqrt(rowSums((trunk$d[,3:5] - 
                            data.frame(X=rep(cb$TargetMc.1[i],nrow(trunk$d[,3:5])),
                                       Y=rep(cb$TargetMc.2[i],nrow(trunk$d[,3:5])),
                                       Z=rep(cb$TargetMc.3[i],nrow(trunk$d[,3:5]))))^2))
  dist2s[i]<-sqrt(rowSums((trunk$d[1,3:5] - 
                             data.frame(X=cb$TargetMc.1[i],
                                        Y=cb$TargetMc.2[i],
                                        Z=cb$TargetMc.3[i])))^2)
  psedo_bid<-which(dist2tmc==min(dist2tmc))
  if(psedo_bid!=1){
    psedo_bts[i]<-sum(sqrt(rowSums((trunk$d[2:psedo_bid,3:5]-trunk$d[match(trunk$d$Parent[2:psedo_bid],trunk$d$PointNo),3:5])^2)))
  }else{
    psedo_bts[i]<-0
  }
}
tmp<-cor.test(cb$PathDisttoSoma,psedo_bts)
lm.data<-lm(y~x,data=data.frame(x=cb$PathDisttoSoma/1000,y=psedo_bts/1000))
pdf(paste0(FIGUREDIR,"Figure 4g.pdf"),width=4,height = 4)
par(mar=c(5.1,6.1,4.1,2.1))
plot(cb$PathDisttoSoma/1000,psedo_bts/1000,pch=19,cex=0.1,col=cb$Color,
     main=paste0("R-squared: ",format(tmp$estimate^2,digits = 2)),font.main=1,
     xlab="Position of branch points on primary axons (mm)",ylab="Nearest points to targets on \nprimary axons (mm)",cex.lab=1)
lines(cb$PathDisttoSoma/1000,predict(lm.data),col="black",lwd=3)
dev.off()

##################
####Figure 4h
Targetregion<-"SCm"
CC<-paste0(c("4"),"-",Targetregion)

##the relationship between projection strength and primary axon position to target region
Length<-c()
dist_p2T<-c()
cc<-c()
cb<-subset(Collateral,TargetRegion==strsplit(CC,"-")[[1]][2]&CSubclass==strsplit(CC,"-")[[1]][1])
subtrunks<-subset(Paxons,names(Paxons) %in% unique(cb$NeuronName))
  
COV = cov.wt(cb[,c("TargetMc.1","TargetMc.2","TargetMc.3")],cb$Length,cor = T,method = "ML")
TargetMc<-COV$center
mean_disttosoma<-mean(cb$PathDisttoSoma)
Planeid<-sPlanes[which(abs(Mean_disttosoma-mean_disttosoma)==min(abs(Mean_disttosoma[which(!is.na(Mean_disttosoma))]-mean_disttosoma)))]
  
pointspos=data.frame(X=rep(NA,length(Paxons)),Y=rep(NA,length(Paxons)),Z=rep(NA,length(Paxons)))
for(i in 1:length(Paxons)){
  pointspos[i,]=Paxons[[i]]$d[Alignmentid[Planeid,i],3:5]
}
dist_p2T<-c(dist_p2T,sqrt(rowSums((pointspos - t(matrix(TargetMc,nrow=3,ncol = nrow(pointspos))))^2)))
  
for(i in 1:length(Paxons)){
  Length<-c(Length,sum(cb$Length[which(cb$NeuronName==names(Paxons)[i])]))
  cc<-c(cc,CC)
}

withcollateral<-rep("Projecting",length(Length))
withcollateral[which(Length==0)]<-"Non-projecting"
ggboxplot(data.frame(WithCollateral=as.factor(withcollateral),Dist=dist_p2T/1000,CSubclass=cc),x="WithCollateral",y="Dist",fill="WithCollateral")+
  stat_compare_means(comparisons = list(c("Projecting","Non-projecting")),method="wilcox.test",label="p.signif",tip.length = 0,size=7)+
  labs(y=paste0("Distance of intersection \npoints to ",Targetregion," (mm)"))+
  theme(axis.text.x = element_text(size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_blank(),axis.title.y =element_text(size=7),
        legend.position = "none")+
  scale_fill_manual(values=c("grey","red"))
ggsave(paste0(FIGUREDIR,"Figure 4h.pdf"),device = "pdf",width = 30,height = 30,scale=1,units = "mm")

##3D plot
open3d()
plot3d(subset(Paxons,!names(Paxons) %in% unique(cb$NeuronName)),col="grey",add=T,soma=70,lwd=0.5)
plot3d(subtrunks,col="red",add=T,soma=70,lwd=2)
for(i in order(cb$Length,decreasing = T)){
  neuron<-Neurons[[which(names(Neurons)==cb$NeuronName[i])]]
  d=neuron$d[cb$Pointlist[[i]],]
  tmp=matrix(NA,nrow=2*nrow(d),ncol=3)
  tmp[seq(1,nrow(tmp),by=2),]<-as.matrix(neuron$d[d$PointNo,3:5])
  tmp[seq(2,nrow(tmp),by=2),]<-as.matrix(neuron$d[d$Parent,3:5])
  segments3d(tmp,col="red",lwd=0.5)
}
shade3d(readOBJ(paste0(FDIR,strsplit(CC,"-")[[1]][2],".obj")),color="grey",override = T,alpha=0.3)

A=as.numeric(V[Planeid,1]/sqrt(sum(V[Planeid,]^2)))
B=as.numeric(V[Planeid,2]/sqrt(sum(V[Planeid,]^2)))
C=as.numeric(V[Planeid,3]/sqrt(sum(V[Planeid,]^2)))
D=as.numeric(-sum(V[Planeid,]*CP[Planeid,])/sqrt(sum(V[Planeid,]^2)))
planes3d(A,B,C,D,col="red",alpha=0.1)

pointspos=data.frame(X=rep(NA,length(Paxons)),
                     Y=rep(NA,length(Paxons)),
                     Z=rep(NA,length(Paxons)))
for(i in 1:length(Paxons)){
  pointspos[i,]=Paxons[[i]]$d[Alignmentid[Planeid,i],3:5]
}
points3d(subset(pointspos,names(Paxons) %in% unique(cb$NeuronName)),col="red",size=20)
points3d(subset(pointspos,!names(Paxons) %in% unique(cb$NeuronName)),col="grey",size=10)

par3d(windowRect = c(103,202,1396,1051),zoom=.6,userMatrix=matrix(c(1,0,0,0,0,-1,0,0,0,0,-1,0,0,0,0,1),4,4))
rgl.snapshot(paste0(FIGUREDIR,"Figure 4h.png"))
rgl.close()

##################
####Figure 4i
brainarea<-c("3-STR","8-P")

neuronnames<-intersect(Collateral$NeuronName[which(paste0(Collateral$CSubclass,"-",Collateral$TargetRegion) %in% brainarea[1])],Collateral$NeuronName[which(paste0(Collateral$CSubclass,"-",Collateral$TargetRegion) %in% brainarea[2])])
cb<-subset(Collateral,NeuronName %in% neuronnames&paste0(Collateral$CSubclass,"-",Collateral$TargetRegion) %in% brainarea)
subtrunks<-subset(Paxons,names(Paxons) %in% unique(cb$NeuronName))

#selected color
dimension<-"Y"
col_fun<-colorRamp2(c(-1,1),c("blue","red"))
color<-col_fun(seq(-1,1,length.out = length(subtrunks)))[order(order(unlist(lapply(subtrunks,function(x)x$d[1,dimension]))))]
names(color)<-names(subtrunks)

#save color bar
pdf(paste0(FIGUREDIR,"Figure 4i (Colorbar).pdf"),width=4,height = 2)
plot_col(length(color),color[order(unlist(lapply(subtrunks,function(x)x$d[1,dimension])))])
dev.off()

open3d()
plot3d(subtrunks,WithNodes = F,soma = 100,NeuronName=F,add=T,col=color,lwd=1)
plot3d(cb$TargetMc.1,cb$TargetMc.2,cb$TargetMc.3,add=T,col=color[match(cb$NeuronName,names(color))],size=15)
tmp=matrix(NA,nrow=2*nrow(cb),ncol=3)
tmp[seq(1,nrow(tmp),by=2),]<-c(cb$BPos.X,cb$BPos.Y,cb$BPos.Z)
tmp[seq(2,nrow(tmp),by=2),]<-c(cb$TargetMc.1,cb$TargetMc.2,cb$TargetMc.3)
segments3d(tmp,col=rep(color[match(cb$NeuronName,names(color))],each=2),lwd=1)

for(b in brainarea){
  wire3d(readOBJ(paste0(FDIR,strsplit(b,"-")[[1]][2],".obj")),color="grey",override = T,alpha=0.1)
}
par3d(windowRect = c(103,202,1396,1051),zoom=0.5,userMatrix=matrix(c(1,0,0,0,0,-1,0,0,0,0,-1,0,0,0,0,1),4,4))
rgl.snapshot(paste0(FIGUREDIR,"Figure 4i.png"))

##################
####Figure 4j
b<-"3-STR"
cb<-subset(Collateral,TargetRegion==strsplit(b,"-")[[1]][2]&CSubclass==strsplit(b,"-")[[1]][1])
subtrunks<-subset(Paxons,names(Paxons) %in% unique(cb$NeuronName))

#select color
dimension<-"Y"
col_fun<-colorRamp2(c(-1,1),c("blue","red"))
color<-col_fun(seq(-1,1,length.out = length(subtrunks)))[order(order(unlist(lapply(subtrunks,function(x)x$d[1,dimension]))))]
names(color)<-names(subtrunks)

xx<-cb$TargetMc.2/1000
yy<-cb$SomaPos.Y/1000
tmp<-cor.test(xx,yy)
pdf(paste0(FIGUREDIR,"Figure 4j (topleft).pdf"),width=3,height = 3)
lm.data<-lm(y~x,data=data.frame(x=xx,y=yy))
par(mar=c(4.1,5.1,4.1,2.1))
plot(xx,yy,pch=19,cex=0.1,col=color[match(cb$NeuronName,names(color))],
     xlim=c(max(xx),min(xx)),ylim=c(max(yy),min(yy)),
     main=paste0("R-squared: ",format(tmp$estimate^2,digits = 2)),font.main=1,cex.main=1,
     xlab="Target center position (mm)",ylab="Soma position (mm)",cex.lab=1)
lines(xx,predict(lm.data),col="black")
dev.off()

xx<-cb$TargetMc.2/1000
yy<-cb$BPos.Y/1000
tmp<-cor.test(xx,yy)
pdf(paste0(FIGUREDIR,"Figure 4j (bottomleft).pdf"),width=3,height = 3)
lm.data<-lm(y~x,data=data.frame(x=xx,y=yy))
par(mar=c(4.1,5.1,4.1,2.1))
plot(xx,yy,pch=19,cex=0.1,col=color[match(cb$NeuronName,names(color))],
     xlim=c(max(xx),min(xx)),ylim=c(max(yy),min(yy)),
     main=paste0("R-squared: ",format(tmp$estimate^2,digits = 2)),font.main=1,cex.main=1,
     xlab="Target center position (mm)",ylab="Branch point position (mm)",cex.lab=1)
lines(xx,predict(lm.data),col="black")
dev.off()

b<-"8-P"
cb<-subset(Collateral,TargetRegion==strsplit(b,"-")[[1]][2]&CSubclass==strsplit(b,"-")[[1]][1])
subtrunks<-subset(Paxons,names(Paxons) %in% unique(cb$NeuronName))

#select color
dimension<-"Y"
col_fun<-colorRamp2(c(-1,1),c("blue","red"))
color<-col_fun(seq(-1,1,length.out = length(subtrunks)))[order(order(unlist(lapply(subtrunks,function(x)x$d[1,dimension]))))]
names(color)<-names(subtrunks)

xx<-cb$TargetMc.2/1000
yy<-cb$SomaPos.Y/1000
tmp<-cor.test(xx,yy)
pdf(paste0(FIGUREDIR,"Figure 4j (topright).pdf"),width=3,height = 3)
lm.data<-lm(y~x,data=data.frame(x=xx,y=yy))
par(mar=c(4.1,5.1,4.1,2.1))
plot(xx,yy,pch=19,cex=0.1,col=color[match(cb$NeuronName,names(color))],
     xlim=c(max(xx),min(xx)),ylim=c(max(yy),min(yy)),
     main=paste0("R-squared: ",format(tmp$estimate^2,digits = 2)),font.main=1,cex.main=1,
     xlab="Target center position (mm)",ylab="Soma position (mm)",cex.lab=1)
lines(xx,predict(lm.data),col="black")
dev.off()

xx<-cb$TargetMc.2/1000
yy<-cb$BPos.Y/1000
tmp<-cor.test(xx,yy)
pdf(paste0(FIGUREDIR,"Figure 4j (bottomright).pdf"),width=3,height = 3)
lm.data<-lm(y~x,data=data.frame(x=xx,y=yy))
par(mar=c(4.1,5.1,4.1,2.1))
plot(xx,yy,pch=19,cex=0.1,col=color[match(cb$NeuronName,names(color))],
     xlim=c(max(xx),min(xx)),ylim=c(max(yy),min(yy)),
     main=paste0("R-squared: ",format(tmp$estimate^2,digits = 2)),font.main=1,cex.main=1,
     xlab="Target center position (mm)",ylab="Branch point position (mm)",cex.lab=1)
lines(xx,predict(lm.data),col="black")
dev.off()
