##################
###load packages
library(WGCNA)
library(pheatmap)
library(circlize)
library(factoextra)
library(nat)
library(qgraph)
library(ggalluvial)
library(magick)

###load data
DATADIR<-"~/课题/Work\ report/Code/Data/"
FDIR<-"~/课题/Work\ report/Code/ForFigures/"
FIGUREDIR<-"~/课题/Work\ report/Code/Figures/"

load(paste0(DATADIR,"allenarbors"))
load(paste0(DATADIR,"Neurons"))

source(paste0(FDIR,"ForFigures.R"))
##################
####Figure 5a
allenarbors<-subset(allenarbors,Type %in% c("ITc","ITi")&annots %in% calannot("CTXpl")&!somalaminar==""&Lw!=0)

# exclude self-projection
delete<-which(allenarbors[,]$cubeid!=0&allenarbors[,]$cubeid==allenarbors[,]$soma_cubeid)
allenarbors<-subset(allenarbors,-delete)

#calculate the length of arbors in each layer 
laminar=c("1","2/3","4","5","6")
allenarbors[,paste0("L",laminar)]<-0
for(i in 1:length(allenarbors)){
  print(i)
  for(l in laminar){
    pl=setdiff(grep(l,allenarbors[[i]]$d$annot),grep("(CA1|CA2|CA3)",allenarbors[[i]]$d$annot))
    pl=pl[which(pl!=1)]
    if(length(pl)>0){
      if(l!="2/3"){
        allenarbors[i,paste0("L",l)]<-sum(sqrt(rowSums((allenarbors[[i]]$d[pl,3:5]-
                                                          allenarbors[[i]]$d[match(allenarbors[[i]]$d$Parent[pl],allenarbors[[i]]$d$PointNo),3:5])^2)))
      }else{
        allenarbors[i,"L2.3"]<-sum(sqrt(rowSums((allenarbors[[i]]$d[pl,3:5]-
                                                   allenarbors[[i]]$d[match(allenarbors[[i]]$d$Parent[pl],allenarbors[[i]]$d$PointNo),3:5])^2)))
      }
    }
  }
}
allenarbors=subset(allenarbors,L1!=0|L2.3!=0|L4!=0|L5!=0|L6!=0)

statistics<-data.frame(allenarbors)[,c("L1","L2.3","L4","L5","L6")]
for(i in 1:nrow(statistics)){
  statistics[i,]<-statistics[i,]/sum(as.numeric(statistics[i,]))
}

allenarbors[,]$color<-NA
allenarbors[,]$Laminar<-NA

out.dist<-get_dist(statistics,method="pearson")
out.hclust<-hclust(out.dist,method="ward.D")
tmp<-cutreeHybrid(out.hclust,as.matrix(out.dist))
Color<-c("#80FF00FF","#00FFFFFF","#FF0000FF","#8000FFFF")

allenarbors[,]$color<-Color[match(tmp$labels,unique(tmp$labels))]
allenarbors[,]$Laminar[which(tmp$labels=="1")]<-"L5"
allenarbors[,]$Laminar[which(tmp$labels=="2")]<-"L2/3"
allenarbors[,]$Laminar[which(tmp$labels=="3")]<-"L1"
allenarbors[,]$Laminar[which(tmp$labels=="4")]<-"L6"
allenarbors[,]$Laminar<-as.factor(allenarbors[,]$Laminar)
allenarbors[,]$somalaminar[which(allenarbors[,]$somalaminar=="1")]<-"2/3"

#find cortical area with L4
areas_4<-c()
for(i in calannot("CTXpl",2)){
  if(any(grepl("4",calannot(i)))){
    areas_4<-c(areas_4,calannot(i))
  }
}

for(i in 1:length(allenarbors)){
  if(all(!allenarbors[[i]]$d$annot %in% areas_4)){
    statistics[i,"L4"]<-NA
  }
}

annotation_col=data.frame(Subclass=as.factor(tmp$labels))
rownames(annotation_col)=rownames(statistics)
ann_colors = list(Subclass=Color[match(1:max(tmp$labels),unique(tmp$labels))])
names(ann_colors$Subclass)<-1:max(as.numeric(annotation_col$Subclass))
col_fun<-colorRamp2(c(0,0.5,1),c("blue","white","red"))
pheatmap(t(as.matrix(statistics[,c("L1","L2.3","L5","L6")])),
         cutree_cols = length(ann_colors$Subclass),cluster_cols = out.hclust,cluster_rows=F,
         fontsize = 7,labels_row = c("L1","L2/3","L5","L6"),show_colnames = F,
         annotation_col = annotation_col,annotation_colors = ann_colors,annotation_names_col = F,
         legend_breaks = c(0,1),
         color = col_fun(seq(0,1,length.out = 100)),na_col = "black",annotation_legend = F,
         main=paste0("Classification of arbors in cortical target areas (n=",prettyNum(length(allenarbors),big.mark = ","),")"),
         filename =paste0(FIGUREDIR,"Figure 5a (heatmap).pdf"),width=6,height = 2)

##plot examples
arbors<-neuronlist()
xmin<-0
xin<-0.1#x axis interval for each arbor
cin<-0.5#x axis interval for each arbor subclass
samplenum<-3
for(l in unique(allenarbors[,]$Laminar[out.hclust$order])){

  if(l=="L1"){
    s<-c(13089,4078,3444)
  }else if(l=="L2/3"){
    s<-c(6799,2053,11215)
  }else if(l=="L6"){
    s<-c(237,15150,398)
  }else if(l=="L5"){
    s<-c(9408,14494,14938)
  }

  for(i in s){
    arbor<-allenarbors[[i]]
    fit<-lm(arbor$d$anno_W~arbor$d$X+arbor$d$Y+arbor$d$Z)
    V<-matrix(c(fit$coefficients[2],fit$coefficients[3],fit$coefficients[4]),3,1)
    v2<-matrix(c(0,1,0),3,1)#the y axis
    v1<-matrix(unlist(V/sqrt(sum(V^2))),3,1)#the w axis
    theta<-acos(t(v1) %*% v2/sqrt(sum(v1^2)*sum(v2^2)))#the angle between v1 and v2
    v3<-rbind(v1[2]*v2[3]-v1[3]*v2[2],v1[3]*v2[1]-v1[1]*v2[3],v1[1]*v2[2]-v1[2]*v2[1])#the vector normal to v1 and v2
    v3<-v3/sqrt(sum(v3^2))
    Rmatx<-as.matrix(rbind(c(v3[1]^2+(1-v3[1]^2)*cos(theta),v3[1]*v3[2]*(1-cos(theta))+v3[3]*sin(theta),v3[1]*v3[3]*(1-cos(theta))-v3[2]*sin(theta),0),
                           c(v3[1]*v3[2]*(1-cos(theta))-v3[3]*sin(theta),v3[2]^2+(1-v3[2]^2)*cos(theta),v3[2]*v3[3]*(1-cos(theta))+v3[1]*sin(theta),0),
                           c(v3[1]*v3[3]*(1-cos(theta))+v3[2]*sin(theta),v3[2]*v3[3]*(1-cos(theta))-v3[1]*sin(theta),v3[3]^2+(1-v3[3]^2)*cos(theta),0),
                           c(0,0,0,1)))#rotate matrix
    tmp<-cbind(arbor$d$X,arbor$d$Y,arbor$d$Z,1) %*% Rmatx
    arbor$d[,3:5]<-tmp[,1:3]
    max_anno_W<-max(arbor$d$anno_W[which(arbor$d$anno_W<1)])
    min_anno_W<-min(arbor$d$anno_W[which(arbor$d$anno_W>0)])
    if(l=="L1"){
      min_anno_W<-min(arbor$d$anno_W)
    }else if(l=="L6a"){
      max_anno_W<-max(arbor$d$anno_W)
    }
    arbor$d$X<-(arbor$d$X-min(arbor$d$X))/(max(arbor$d$Y)-min(arbor$d$Y))*(max_anno_W-min_anno_W)+xmin
    arbor$d$Z<-(arbor$d$Z-min(arbor$d$Z))/(max(arbor$d$Y)-min(arbor$d$Y))*(max_anno_W-min_anno_W)
    arbor$d$Y<-(arbor$d$Y-min(arbor$d$Y))/(max(arbor$d$Y)-min(arbor$d$Y))*(max_anno_W-min_anno_W)+min_anno_W
    xmin<-max(arbor$d$X)+xin
    arbor<-as.neuronlist(arbor)
    names(arbor)<-names(allenarbors)[i]
    data.frame(arbor)<-data.frame(allenarbors)[i,]
    arbors<-nat::union(arbors,as.neuronlist(arbor))
  }
  xmin<-xmin+cin
}

pdf(paste0(FIGUREDIR,"Figure 5a (example arbors).pdf"),width=6,height = 2)
par(mar=c(2.1,5.1,4.1,4.1))
plot(arbors,ylim=c(1,0),asp=NA,WithNodes=F,col=arbors[,]$color,main="",xlab="",ylab="",tck=0,axes=F,lwd=0.25)
box(lwd=0.5)
axis(side=4,las=2,cex.axis=0.7,lwd=0.5,tck=-0.05)
axis(side=2,at=c(0.05,0.16,0.41,0.8),labels = c("L1","L2/3","L5","L6"),lwd.ticks = 0,las=2,cex.axis=0.7,lwd=0.5)
mtext(4,text="Relative depth",line = 2,cex=0.7)
abline(h=c(0,0.1,0.23,0.6,1),col="black",lty=2,lwd=0.5)
dev.off()

##################
####Figure 5b,c
###Figure 5b
##relationship between soma laminar and arbor type usage
subarbors<-subset(allenarbors,somaposition %in% c("ORBvl","ORBl","ORBm","AId","AIv","ACAd","ACAv","PL","FRP","ILA","MOs")&RL=="L"&somalaminar %in% c("2/3","5","6a"))

#exclude self-projection
delete<-c()
for(s in unique(subarbors[,]$somaposition)){
  delete<-c(delete,which(subarbors[,]$annots %in% calannot(s)&subarbors[,]$somaposition==s))
}
subarbors<-subset(subarbors,-delete)

arborsfreq<-c()
somalaminar<-c()
arborlaminar<-c()
for(i in unique(subarbors[,]$somalaminar[which(subarbors[,]$somalaminar %in% c("1","2/3","5","6a"))])){
  for(j in unique(subarbors[,]$Laminar)){
    arborsfreq<-c(arborsfreq,length(which(subarbors[,]$somalaminar==i&subarbors[,]$Laminar==j))/length(which(subarbors[,]$somalaminar==i)))
    somalaminar<-c(somalaminar,i)
    arborlaminar<-c(arborlaminar,j)
  }
}
ggplot(data.frame(Proportion=arborsfreq,SomaLaminar=as.factor(somalaminar),ArborLaminar=as.factor(arborlaminar)), aes(x =SomaLaminar , y = Proportion,fill=ArborLaminar)) +
  geom_bar(stat = "identity", position = "stack")+
  scale_fill_manual(values=unique(subarbors[,]$color)[match(levels(subarbors[,]$Laminar),unique(subarbors[,]$Laminar))],name="Arbor type",labels=c("1(L1)","2(L2/3)","3(L5)","4(L6)"))+
  scale_x_discrete(labels=c("L2/3","L5","L6"))+
  scale_y_continuous(n.breaks = 2)+
  labs(x="Soma (cortical layer)")+
  theme_classic()+
  theme(plot.title = element_text(hjust = 0.5,size=8),
        axis.text.x = element_text(vjust = 0.5,size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_text(size=8),axis.title.y =element_text(size=8),
        legend.title = element_text(hjust = 0.5,size=8),legend.text = element_text(hjust = 0.5,size=8),
        legend.key.height=unit(2.5,"mm"),legend.key.width = unit(2,"mm"),legend.text.align=0)
ggsave(paste0(FIGUREDIR,"Figure 5b (left).pdf"),device = "pdf",width = 40,height = 40,units = "mm")

##relationship between soma relative depth and terminal arbor relative depth
xx=allenarbors[,]$mean_W
yy=allenarbors[,]$soma_W
yy[which(is.na(yy))]<-0
tmp=cor.test(xx,yy)
pdf(paste0(FIGUREDIR,"Figure 5b (right).pdf"),width=4,height = 4)
par(mar=c(4.1,5.1,4.1,2.1))
plot(xx,yy,pch=19,cex=0.1,col="blue",
     main=paste0("R-squared: ",format(tmp$estimate^2,digits = 2)),font.main=1,cex.main=1,
     ylab="Soma relative depth",xlab="Terminal arbor relative depth",cex.lab=1)
lm.data=lm(y~x,data=data.frame(x=as.vector(xx),y=as.vector(yy)))
lines(xx,predict(lm.data),col="red",lwd=2)
dev.off()

###Figure 5c
##relationship between arbor type usage and soma regions
arborsfreq<-c()
somaregion<-c()
arborlaminar<-c()
arbornum<-c()
for(i in unique(subarbors[,]$somaposition)){
  for(j in unique(subarbors[,]$Laminar)){
    arborsfreq=c(arborsfreq,length(which(subarbors[,]$somaposition==i&subarbors[,]$Laminar==j))/length(which(subarbors[,]$somaposition==i)))
    somaregion=c(somaregion,i)
    arborlaminar=c(arborlaminar,j)
    arbornum<-c(arbornum,length(which(subarbors[,]$somaposition==i)))
  }
}

Data<-data.frame(Proportion=arborsfreq,SomaRegion=factor(somaregion,levels=unique(somaregion)[order(arborsfreq[which(arborlaminar=="L1")])]),ArborLaminar=as.factor(arborlaminar),ArborNum=arbornum)
Data<-subset(Data,ArborNum>10)
ggplot(Data, aes(x =SomaRegion , y = Proportion,fill=ArborLaminar)) +
  geom_bar(stat = "identity", position = "stack")+
  scale_fill_manual(values=unique(subarbors[,]$color)[match(levels(subarbors[,]$Laminar),unique(subarbors[,]$Laminar))])+
  scale_y_continuous(n.breaks = 2)+
  labs(x="Soma region")+
  theme_classic()+
  theme(axis.text.x = element_text(vjust = 0.5,angle = 80,size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_text(size=7),axis.title.y =element_text(size=7),
        legend.position = "none")
ggsave(paste0(FIGUREDIR,"Figure 5c (left).pdf"),device = "pdf",width = 35,height = 40,units = "mm")

##relationship between arbor type usage and target regions
brainarea<-c("ORBvl","ORBl","ORBm","AId","AIv","ACAd","ACAv","PL","FRP","ILA","MOs","MOp","SSs","SSp","GU","VISC","AUD","VIS","RSP","PTLp","TEa","PERI","ECT","PFC","non-PFC")
arborsfreq<-c()
targetregion<-c()
arborlaminar<-c()
arbornum<-c()
for(i in brainarea){
  for(j in unique(subarbors[,]$Laminar)){
    if(!i %in% c("PFC","non-PFC")){
      arborsfreq<-c(arborsfreq,length(which(subarbors[,]$annots %in% calannot(i)&subarbors[,]$Laminar==j))/length(which(subarbors[,]$annots %in% calannot(i))))
      arbornum<-c(arbornum,length(which(subarbors[,]$annots %in% calannot(i))))
      targetregion<-c(targetregion,i)
      arborlaminar<-c(arborlaminar,j)
    }else if(i=="PFC"){
      brainregions<-c()
      for(b in c("ORBvl","ORBl","ORBm","AId","AIv","ACAd","ACAv","PL","FRP","ILA","MOs")){
        brainregions<-c(brainregions,calannot(b))
      }
      arborsfreq<-c(arborsfreq,length(which(subarbors[,]$annots %in% brainregions&subarbors[,]$Laminar==j))/length(which(subarbors[,]$annots %in% brainregions)))
      arbornum<-c(arbornum,length(which(subarbors[,]$annots %in% brainregions)))
      targetregion<-c(targetregion,i)
      arborlaminar<-c(arborlaminar,j)
    }else if(i=="non-PFC"){
      brainregions<-c()
      for(b in c("MOp","SSs","SSp","GU","VISC","AUD","VIS","RSP","PTLp","TEa","PERI","ECT")){
        brainregions<-c(brainregions,calannot(b))
      }
      arborsfreq<-c(arborsfreq,length(which(subarbors[,]$annots %in% brainregions&subarbors[,]$Laminar==j))/length(which(subarbors[,]$annots %in% brainregions)))
      arbornum<-c(arbornum,length(which(subarbors[,]$annots %in% brainregions)))
      targetregion<-c(targetregion,i)
      arborlaminar<-c(arborlaminar,j)
    }
  }
}

o<-order(arborsfreq[which(arborlaminar=="L5")])
o<-o[c(which(unique(targetregion)[o] %in% c("MOp","SSs","SSp","GU","VISC","AUD","VIS","RSP","PTLp","TEa","PERI","ECT")),
         which(unique(targetregion)[o] %in% c("ORBvl","ORBl","ORBm","AId","AIv","ACAd","ACAv","PL","FRP","ILA","MOs")),
         which(unique(targetregion)[o] %in% c("PFC","non-PFC")))]
color<-rep("black",length(unique(targetregion)))
color[which(unique(targetregion)[o] %in% c("ORBvl","ORBl","ORBm","AId","AIv","ACAd","ACAv","PL","FRP","ILA","MOs","PFC"))]<-"#C30D23"
Data<-data.frame(Proportion=arborsfreq,TargetRegion=factor(targetregion,levels=unique(targetregion)[o]),ArborLaminar=as.factor(arborlaminar),ArborNum=arbornum)
Data<-subset(Data,ArborNum>10)
ggplot(Data, aes(x =TargetRegion , y = Proportion,fill=ArborLaminar)) +
  geom_bar(stat = "identity", position = "stack")+
  scale_y_continuous(n.breaks = 2)+
  scale_fill_manual(values=unique(subarbors[,]$color)[match(levels(subarbors[,]$Laminar),unique(subarbors[,]$Laminar))],name="Arbor type",labels=c("1(L1)","2(L2/3)","3(L5)","4(L6)"))+
  labs(x="Target region")+
  theme_classic()+
  theme(axis.text.x = element_text(vjust = 0.5,angle = 80,size=7,color = color),axis.text.y =element_text(size=7),
        axis.title.x =element_text(size=7),axis.title.y =element_text(size=7),
        legend.title = element_text(hjust = 0.5,size=8),legend.text = element_text(hjust = 0.5,size=8),
        legend.key.height=unit(2.5,"mm"),legend.key.width = unit(2,"mm"),legend.text.align=0)
ggsave(paste0(FIGUREDIR,"Figure 5c (right).pdf"),device = "pdf",width = 70,height = 40,units = "mm")

##################
####Figure 5d,e
###Figure 5d
##the laminar difference of output-input between PFC cubes
subarbors<-subset(allenarbors,soma_cubeid!="0"&RL=="L"&somalaminar %in% c("2/3","5","6a"))

#replace PFC regions with cube id
targetregions<-c()
for(i in c("ORBvl","ORBl","ORBm","AId","AIv","ACAd","ACAv","PL","FRP","ILA","MOs")){
  targetregions<-c(targetregions,calannot(i))
}
subarbors[,]$annots[which(subarbors[,]$annots %in% targetregions)]<-subarbors[,]$cubeid[which(subarbors[,]$annots %in% targetregions)]
subarbors[,]$somaposition<-as.character(subarbors[,]$somaposition)
subarbors[,]$somaposition[which(subarbors[,]$somaposition %in% targetregions)]<-subarbors[,]$soma_cubeid[which(subarbors[,]$somaposition %in% targetregions)]

#exclude soma regions with few neurons
tmp<-subarbors[,c("somaposition","somalaminar","NeuronName")]
tmp<-unique(tmp)
tmp<-summary(as.factor(tmp$somaposition))
brainarea<-names(tmp)[which(tmp>10)]
brainarea<-brainarea[which(brainarea %in% c("15","30","34","22","16","25","19","9","1","26","20","36","2","7","38","23","37","32","24","33","10","39","13","40","35","6","11","28","14","12","18"))]
targetregion<-c(unique(subarbors[,]$soma_cubeid))
targetregion<-targetregion[which(targetregion %in% c("15","30","34","22","16","25","19","9","1","26","20","36","2","7","38","23","37","32","24","33","10","39","13","40","35","6","11","28","14","12","18"))]

ll<-unique(paste0("L",subarbors[,]$somalaminar,"->",subarbors[,]$Laminar))
ll<-ll[order(ll)]
pathway<-c()
arborlaminar<-c()
arbornum<-c()
arbornum_tl<-c()#total arbor number in different laminar of different target region
arbornum_sl<-c()#total arbor number from different laminar of different soma region
arbornum_s<-c()#total arbor number from different soma region
for(tt in targetregion){
    for(i in brainarea){
      for(j in ll){
        somalayer<-strsplit(j,"->")[[1]][1]
        somalayer<-substr(somalayer,2,nchar(somalayer))
        targetlayer<-strsplit(j,"->")[[1]][2]
        
        if(tt %in% unique(subarbors[,]$soma_cubeid)){
          arbornum_tl<-c(arbornum_tl,length(which(subarbors[,]$somaposition==i&subarbors[,]$annots==tt&subarbors[,]$Laminar==targetlayer)))
          arbornum_sl<-c(arbornum_sl,length(which(subarbors[,]$somaposition==i&subarbors[,]$annots==tt&subarbors[,]$somalaminar==somalayer)))
          arbornum_s<-c(arbornum_s,length(which(subarbors[,]$somaposition==i&subarbors[,]$annots==tt)))
          arbornum<-c(arbornum,length(which(subarbors[,]$somaposition==i&paste0("L",subarbors[,]$somalaminar,"->",subarbors[,]$Laminar)==j&subarbors[,]$annots==tt)))
        }else{
          arbornum_tl<-c(arbornum_tl,length(which(subarbors[,]$somaposition==i&subarbors[,]$annots %in% calannot(tt)&subarbors[,]$Laminar==targetlayer)))
          arbornum_sl<-c(arbornum_sl,length(which(subarbors[,]$somaposition==i&subarbors[,]$annots %in% calannot(tt)&subarbors[,]$somalaminar==somalayer)))
          arbornum_s<-c(arbornum_s,length(which(subarbors[,]$somaposition==i&subarbors[,]$annots %in% calannot(tt))))
          arbornum<-c(arbornum,length(which(subarbors[,]$somaposition==i&paste0("L",subarbors[,]$somalaminar,"->",subarbors[,]$Laminar)==j&subarbors[,]$annots %in% calannot(tt))))
        }
        
        pathway<-c(pathway,paste0(i,"->",tt))
        arborlaminar<-c(arborlaminar,j)
      }
    }
  }

Data<-data.frame(Pathway=pathway,ArborLaminar=as.factor(arborlaminar),Arbornum=arbornum,Arbornum_tl=arbornum_tl,Arbornum_sl=arbornum_sl,Arbornum_s=arbornum_s)

tmp<-Data[,c("Pathway","ArborLaminar","Arbornum_tl","Arbornum_s")]
tmp$ArborLaminar<-unlist(lapply(strsplit(as.character(Data$ArborLaminar),"->"),function(x)x[2]))
tmp<-unique(tmp)
tmp<-subset(tmp,Arbornum_s>10)

###plot heatmap
heatmapmatx<-matrix(NA,nrow=length(unique(tmp$ArborLaminar)),ncol=length(unique(tmp$Pathway)),dimnames = list(unique(tmp$ArborLaminar),unique(tmp$Pathway)))
id<-cbind(match(tmp$ArborLaminar,rownames(heatmapmatx)),match(tmp$Pathway,colnames(heatmapmatx)))
heatmapmatx[id]<-tmp$Arbornum_tl/tmp$Arbornum_s

#distinguish soma laminar
tmp<-subset(Data,Pathway %in% colnames(heatmapmatx))
heatmapmatx_l<-matrix(NA,nrow=length(unique(tmp$ArborLaminar)),ncol=length(unique(tmp$Pathway)),dimnames = list(unique(tmp$ArborLaminar),unique(tmp$Pathway)))
id<-cbind(match(tmp$ArborLaminar,rownames(heatmapmatx_l)),match(tmp$Pathway,colnames(heatmapmatx_l)))
heatmapmatx_l[id]<-tmp$Arbornum/tmp$Arbornum_sl

out.dist<-get_dist(t(heatmapmatx),method = "pearson")
out.hclust<-hclust(out.dist,method="ward.D")
tmp<-cutreeHybrid(out.hclust,as.matrix(out.dist),deepSplit = 0,minClusterSize = 10)
Color<-getcol(unique(tmp$labels),colorpattern = "random")
Color[which(unique(tmp$labels)==1)]<-"#00E980"
Color[which(unique(tmp$labels)==2)]<-"orange"
Color[which(unique(tmp$labels)==3)]<-"#FF00DB"
Color[which(unique(tmp$labels)==4)]<-"#0092FF"

heatmapmatx_l<-rbind(heatmapmatx,heatmapmatx_l)

Annotation_col=data.frame(Subclass=as.factor(tmp$labels))
rownames(Annotation_col)=colnames(heatmapmatx)
ann_colors = list(Subclass=Color[match(1:max(tmp$labels),unique(tmp$labels))])
names(ann_colors$Subclass)<-1:max(as.numeric(Annotation_col$Subclass))

##plot mean heatmapmatx
arborlaminar<-unlist(lapply(strsplit(as.character(Data$ArborLaminar),"->"),function(x)x[2]))
meanheatmapmatx<-matrix(NA,nrow(heatmapmatx),max(as.numeric(Annotation_col$Subclass)),dimnames = list(unique(arborlaminar)[order(unique(arborlaminar))],levels(Annotation_col$Subclass)))
for(i in rownames(heatmapmatx)){
  for(j in 1:max(as.numeric(Annotation_col$Subclass))){
    meanheatmapmatx[which(rownames(heatmapmatx)==i),j]<-sum(Data$Arbornum[which(arborlaminar==i&Data$Pathway %in% rownames(Annotation_col)[which(Annotation_col$Subclass==j)])])
  }
}

for(i in 1:ncol(meanheatmapmatx)){
  meanheatmapmatx[,i]<-meanheatmapmatx[,i]/sum(meanheatmapmatx[,i])
}

meanheatmapmatx_l<-meanheatmapmatx

annotation_col=data.frame(Subclass=1:max(as.numeric(Annotation_col$Subclass)))
rownames(annotation_col)=1:max(as.numeric(Annotation_col$Subclass))
col_fun<-colorRamp2(c(min(meanheatmapmatx_l[which(!is.na(meanheatmapmatx_l))]),sum(range(meanheatmapmatx_l[which(!is.na(meanheatmapmatx_l))]))/2,max(meanheatmapmatx_l[which(!is.na(meanheatmapmatx_l))])),c("blue","white","red"))

tmp<-pheatmap(meanheatmapmatx_l,cluster_cols = F,cluster_rows=F,
              annotation_col = annotation_col,annotation_colors = ann_colors,angle_col = 0,
              color = col_fun(seq(min(meanheatmapmatx_l[which(!is.na(meanheatmapmatx_l))]),max(meanheatmapmatx_l[which(!is.na(meanheatmapmatx_l))]),length.out = 100)),
              na_col = "black",display_numbers = T,
              annotation_legend = F,legend_breaks = c(0.1,0.6),
              fontsize_row = 8,fontsize_col = 9,fontsize_number = 7,
              main="Classification of cortical connections \nbased on arbor type usage",
              filename = paste0(FIGUREDIR,"Figure 5d.pdf"),width=3,height = 3)

###Figure 5e
tmp<-strsplit(colnames(heatmapmatx_l),"->")
E<-data.frame(from=unlist(lapply(tmp,function(x)x[1])),to=unlist(lapply(tmp,function(x)x[2])),edge.color=Color[match(Annotation_col$Subclass,unique(Annotation_col$Subclass))],arborlaminar=Annotation_col$Subclass)
E<-E[which(E[,"from"] %in% unique(subarbors[,]$somaposition)&E[,"to"] %in% unique(subarbors[,]$somaposition)),]
E[,"width"]<-NA
for(i in 1:nrow(E)){
  arbornum<-c(length(which(subarbors[,]$somaposition==E[i,"from"]&subarbors[,]$annots==E[i,"to"])),
              length(which(subarbors[,]$somaposition==E[i,"to"]&subarbors[,]$annots==E[i,"from"])))
  if(all(arbornum>10)){
    ma<-rbind(summary(subarbors[,]$Laminar[which(subarbors[,]$somaposition==E[i,"from"]&subarbors[,]$annots==E[i,"to"])]),
              summary(subarbors[,]$Laminar[which(subarbors[,]$somaposition==E[i,"to"]&subarbors[,]$annots==E[i,"from"])]))
    E[i,"width"]<-fisher.test(ma)$p.value
  }
}
E[which(!is.na(E[,"width"])),"width"]<-p.adjust(E[which(!is.na(E[,"width"])),"width"])

#network plot
E[which(!is.na(E[,"width"])),"width"]<--log10(E[which(!is.na(E[,"width"])),"width"])
E[which(is.na(E[,"width"])|E[,"width"]==0),"width"]<-min(E[which(!is.na(E[,"width"])&E[,"width"]!=0),"width"])/2
E[,"width"]<-as.character((as.numeric(E[,"width"])*10))
E<-E[which(E[,"to"] %in% c("15","30","34","22","16","25","19","9","1","26","20","36","2","7","38","23","37","32","24","33","10","39","13","40","35","6","11","28","14","12","18")&
             E[,"from"] %in% c("15","30","34","22","16","25","19","9","1","26","20","36","2","7","38","23","37","32","24","33","10","39","13","40","35","6","11","28","14","12","18")),]

Nodecolor<-rep(NA,length(Nodenames))
Nodecolor[which(Nodenames %in% c("15","30","34","22","16","25","19","9","1","26"))]<-"red"
Nodecolor[which(Nodenames %in% c("20","36","2","7","38","23","37","32","24","33","10"))]<-"green"
Nodecolor[which(Nodenames %in% c("39","13","40","35","6","11","28","14","12","18"))]<-"blue"
Nodecolor[which(Nodenames %in% c("4","17","29","3","31","27","21","5","8"))]<-"grey"

lcolor<-rep("black",length(Nodenames))
lcolor[which(Nodenames %in% c("39","13","40","35","6","11","28","14","12","18"))]<-"white"

Layout<-cbind(c(406,535,673,673,826,734,829,829,716,756,805,867,917,978,983,1063,1138,1073,1095,1145,1017,975,920,1022,1071,1136,1152,1095,1171,1120,1062,975,847,797,891,1006,968,1083,1124,970),
              -c(275,206,190,121,72,205,128,190,285,359,416,472,546,617,715,704,677,611,494,565,521,438,370,343,427,447,349,356,257,275,248,276,328,275,226,189,126,162,128,54))
rownames(Layout)<-c("20","36","2","7","15","38","23","37","4","17","29","31","21","28","5","8","12","14","6","11","18","27","3","39","40","35","13","26","1","9","19","10","33","32","24","22","34","16","25","30")
Nodenames<-unique(c(as.character(E[,"from"]),as.character(E[,"to"])))
Layout<-Layout[match(Nodenames,rownames(Layout)),]

g<-qgraph(E[,c("from","to","width")],layout=Layout,
          edge.color =as.character(E[,"edge.color"]),color=Nodecolor,label.color=lcolor,
          fade=F,parallelEdge=T,
          esize=10,asize=pmax(as.numeric(E[,"width"])/5,2),vsize=4,label.cex=2,
          filename=paste0(FIGUREDIR,"Figure 5e"),filetype="pdf",width=5,height=4)

##################
###Figure 5f
somalaminar<-c("2/3","5","6a")
somaregion<-c("22")
somaregion<-paste0(rep(somaregion,length(somalaminar)),", L",rep(somalaminar,each=length(somaregion)))
targetregion<-c("7")
arborlaminar<-levels(allenarbors[,]$Laminar)

Data<-data.frame(SomaRegion=rep(somaregion,each=length(targetregion)*length(arborlaminar)),
                 TargetRegion=rep(rep(targetregion,each=length(arborlaminar)),length(somaregion)),
                 Laminar=rep(arborlaminar,length(somaregion)*length(targetregion)))
Data$Proportion<-NA
for(ss in somaregion){
  for(tt in targetregion){
    tmp<-summary(subarbors[,]$Laminar[which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar)==ss&subarbors[,]$annots==tt)])
    Data$Proportion[which(Data$SomaRegion==ss&Data$TargetRegion==tt)]<-tmp/length(which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar) %in% somaregion&subarbors[,]$annots %in% targetregion))
  }
}
Data<-subset(Data,Proportion>0)

ggplot(Data,aes(axis1=SomaRegion,axis2=TargetRegion,y=Proportion))+
  scale_x_discrete(limits=c("SomaRegion","TargetRegion"),labels=c("Source","Target"))+
  scale_y_continuous(breaks = c(0,1))+
  geom_alluvium(aes(fill=Laminar),aes.bind = "alluvia")+
  geom_stratum(alpha=0,size=0.25)+ 
  geom_text(stat = "stratum",aes(label = after_stat(stratum)),size=2)+
  scale_fill_manual(values=unique(allenarbors[,]$color)[match(levels(Data$Laminar)[which(levels(Data$Laminar) %in% unique(Data$Laminar))],unique(allenarbors[,]$Laminar))])+
  theme_classic()+
  theme(axis.text.x = element_text(size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_text(size=7),axis.title.y =element_text(size=7),
        axis.line = element_line(size=0.5),
        legend.position = "none")
ggsave(paste0(FIGUREDIR,"Figure 5f(1).pdf"),device = "pdf",width = 40,height = 55,units = "mm")

somalaminar<-c("2/3","5","6a")
somaregion<-c("7")
somaregion<-paste0(rep(somaregion,length(somalaminar)),", L",rep(somalaminar,each=length(somaregion)))
targetregion<-c("22")
arborlaminar<-levels(allenarbors[,]$Laminar)

Data<-data.frame(SomaRegion=rep(somaregion,each=length(targetregion)*length(arborlaminar)),
                 TargetRegion=rep(rep(targetregion,each=length(arborlaminar)),length(somaregion)),
                 Laminar=rep(arborlaminar,length(somaregion)*length(targetregion)))
Data$Proportion<-NA
for(ss in somaregion){
  for(tt in targetregion){
    tmp<-summary(subarbors[,]$Laminar[which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar)==ss&subarbors[,]$annots==tt)])
    Data$Proportion[which(Data$SomaRegion==ss&Data$TargetRegion==tt)]<-tmp/length(which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar) %in% somaregion&subarbors[,]$annots %in% targetregion))
  }
}
Data<-subset(Data,Proportion>0)

ggplot(Data,aes(axis1=SomaRegion,axis2=TargetRegion,y=Proportion))+
  scale_x_discrete(limits=c("SomaRegion","TargetRegion"),labels=c("Source","Target"))+
  scale_y_continuous(breaks = c(0,1))+
  geom_alluvium(aes(fill=Laminar),aes.bind = "alluvia")+
  geom_stratum(alpha=0,size=0.25)+ 
  geom_text(stat = "stratum",aes(label = after_stat(stratum)),size=2)+
  scale_fill_manual(values=unique(allenarbors[,]$color)[match(levels(Data$Laminar)[which(levels(Data$Laminar) %in% unique(Data$Laminar))],unique(allenarbors[,]$Laminar))])+
  theme_classic()+
  theme(axis.text.x = element_text(size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_text(size=7),axis.title.y =element_text(size=7),
        axis.line = element_line(size=0.5),
        legend.position = "none")
ggsave(paste0(FIGUREDIR,"Figure 5f(2).pdf"),device = "pdf",width = 40,height = 55,units = "mm")

somalaminar<-c("2/3","5","6a")
somaregion<-c("34")
somaregion<-paste0(rep(somaregion,length(somalaminar)),", L",rep(somalaminar,each=length(somaregion)))
targetregion<-c("28")
arborlaminar<-levels(allenarbors[,]$Laminar)

Data<-data.frame(SomaRegion=rep(somaregion,each=length(targetregion)*length(arborlaminar)),
                 TargetRegion=rep(rep(targetregion,each=length(arborlaminar)),length(somaregion)),
                 Laminar=rep(arborlaminar,length(somaregion)*length(targetregion)))
Data$Proportion<-NA
for(ss in somaregion){
  for(tt in targetregion){
    tmp<-summary(subarbors[,]$Laminar[which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar)==ss&subarbors[,]$annots==tt)])
    Data$Proportion[which(Data$SomaRegion==ss&Data$TargetRegion==tt)]<-tmp/length(which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar) %in% somaregion&subarbors[,]$annots %in% targetregion))
  }
}
Data<-subset(Data,Proportion>0)

ggplot(Data,aes(axis1=SomaRegion,axis2=TargetRegion,y=Proportion))+
  scale_x_discrete(limits=c("SomaRegion","TargetRegion"),labels=c("Source","Target"))+
  scale_y_continuous(breaks = c(0,1))+
  geom_alluvium(aes(fill=Laminar),aes.bind = "alluvia")+
  geom_stratum(alpha=0,size=0.25)+ 
  geom_text(stat = "stratum",aes(label = after_stat(stratum)),size=2)+
  scale_fill_manual(values=unique(allenarbors[,]$color)[match(levels(Data$Laminar)[which(levels(Data$Laminar) %in% unique(Data$Laminar))],unique(allenarbors[,]$Laminar))],
                    name="Arbor type",labels=c("1(L1)","2(L2/3)","3(L5)","4(L6)"))+
  theme_classic()+
  theme(axis.text.x = element_text(size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_text(size=7),axis.title.y =element_text(size=7),
        axis.line = element_line(size=0.5),
        legend.title=element_text(size=7),legend.text = element_text(size=7),
        legend.key.height=unit(3,"mm"),legend.key.width = unit(2.5,"mm"))
ggsave(paste0(FIGUREDIR,"Figure 5f(3).pdf"),device = "pdf",width = 60,height = 55,units = "mm")

somalaminar<-c("2/3","5","6a")
somaregion<-c("28")
somaregion<-paste0(rep(somaregion,length(somalaminar)),", L",rep(somalaminar,each=length(somaregion)))
targetregion<-c("34")
arborlaminar<-levels(allenarbors[,]$Laminar)

Data<-data.frame(SomaRegion=rep(somaregion,each=length(targetregion)*length(arborlaminar)),TargetRegion=rep(rep(targetregion,each=length(arborlaminar)),length(somaregion)),Laminar=rep(arborlaminar,length(somaregion)*length(targetregion)))
Data$Proportion<-NA
for(ss in somaregion){
  for(tt in targetregion){
    tmp<-summary(subarbors[,]$Laminar[which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar)==ss&subarbors[,]$annots==tt)])
    Data$Proportion[which(Data$SomaRegion==ss&Data$TargetRegion==tt)]<-tmp/length(which(paste0(subarbors[,]$somaposition,", L",subarbors[,]$somalaminar) %in% somaregion&subarbors[,]$annots %in% targetregion))
  }
}
Data<-subset(Data,Proportion>0)

ggplot(Data,aes(axis1=SomaRegion,axis2=TargetRegion,y=Proportion))+
  scale_x_discrete(limits=c("SomaRegion","TargetRegion"),labels=c("Source","Target"))+
  scale_y_continuous(breaks = c(0,1))+
  geom_alluvium(aes(fill=Laminar),aes.bind = "alluvia")+
  geom_stratum(alpha=0,size=0.25)+ 
  geom_text(stat = "stratum",aes(label = after_stat(stratum)),size=2)+
  scale_fill_manual(values=unique(allenarbors[,]$color)[match(levels(Data$Laminar)[which(levels(Data$Laminar) %in% unique(Data$Laminar))],unique(allenarbors[,]$Laminar))])+
  theme_classic()+
  theme(axis.text.x = element_text(size=7),axis.text.y =element_text(size=7),
        axis.title.x =element_text(size=7),axis.title.y =element_text(size=7),
        axis.line = element_line(size=0.5),
        legend.position = "none")
ggsave(paste0(FIGUREDIR,"Figure 5f(4).pdf"),device = "pdf",width = 40,height = 55,units = "mm")

##################
###Figure 5g
somaregion<-c("34")
somalayer<-"5"
targetregion<-c("28")

neuronnames<-names(Neurons)
arbornames<-names(subarbors)
for(tt in targetregion){
  neuronnames<-intersect(neuronnames,unique(subarbors[,]$NeuronName[which(subarbors[,]$annots==tt&subarbors[,]$somaposition==somaregion&subarbors[,]$somalaminar==somalayer&subarbors[,]$RL=="L")]))
}

##plot neurons
s<-c("200199_062","194783_042")

#2D plot
axes<-c("X","Y")
allensize<-c(1320,800,1140)
names(allensize)<-c("X","Y","Z")
bg<-image_read(paste0(FDIR,"450_sagittal.png"))
bg<-image_flip(bg)
pdf(paste0(FIGUREDIR,"Figure 5g.pdf"),width=6,height = 6)
tmp<-subset(Neurons,s)
color<-getcol(1:length(s))
color<-c("red","blue")
plot(bg,xlim=c(0,allensize[which(names(allensize)==axes[1])]),ylim=c(allensize[which(names(allensize)==axes[2])],0))
for(i in 1:length(tmp)){
  tmp[[i]]$d[,3:5]<-tmp[[i]]$d[,3:5]/10
}
plot(tmp,WithNodes=F,soma=10,col=color,add=T,lwd=0.5,PlotAxes=paste0(axes,collapse = ""))
somapos<-cbind(unlist(lapply(tmp,function(x)x$d[1,axes[1]])),unlist(lapply(tmp,function(x)x$d[1,axes[2]])))
points(somapos,lwd=1,bg=color,pch=21)
dev.off()